<?php

namespace Framework;

class Resource extends \Tonic\Resource {
	protected $isPublic = false;

	protected $dataBase;
	protected $baseUri;

	protected $authToken;
	protected $userGroup;
	protected $userId;
	
	protected $restrictedUser;

	protected $filter;
	protected $pageInfo;
	protected $sortInfo;
	
	protected function searchCols() {
		return array();
	}

	protected function filterArgs() {
		return array();
	}

	protected function sortCols() {
		return array();
	}
	
	protected function defaultSortBy() {
		return null;
	}
	
	protected function defaultSortOrder() {
		return null;
	}

	public function exec() {
		$connectionTimeout = ini_get("max_execution_time");

		if ($connectionTimeout == 0 || $connectionTimeout > 10) {
			$connectionTimeout = 10;
		}

		$dbHost = getenv('DB_HOST');
		$dbUser = getenv('DB_USER');
		$dbPass = getenv('DB_PASS');
		$dbName = getenv('DB_NAME');

		$dataBase = new \mysqli();

		$dataBase->options(MYSQLI_OPT_CONNECT_TIMEOUT, $connectionTimeout);
		$dataBase->real_connect($dbHost, $dbUser, $dbPass, $dbName);
		$dataBase->set_charset('utf8');

		$this->dataBase = $dataBase;
		
		$this->baseUri = dirname($_SERVER['PHP_SELF']);
		
		if (!$this->isPublic) {
			$this->authToken = $this->request->xWnmsAuthToken;

			if ($this->authToken != NULL) {
				$statement = $dataBase->prepare('CALL GetSessionUser(?)');
				$statement->bind_param('s', $this->authToken);
				$statement->execute();
				$statement->bind_result($this->userId, $this->userGroup);
				$statement->fetch();
				$statement->close();

				$dataBase->next_result();
				
				$this->restrictedUser = ($this->userGroup === 'NETWORK');
			}

			if ($this->userId == 0) {
				throw new \Tonic\UnauthorizedException;
			}
		}

		$request = $this->request;

		if ($request->contentType == 'application/json') {
			$request->data = json_decode($request->data);
		}

		$this->filter = Filter::create($this->filterArgs(), $this->searchCols());
		
		$this->pageInfo = PageInfo::create();
		$this->sortInfo = SortInfo::create($this->sortCols(),
			$this->defaultSortBy(),
			$this->defaultSortOrder()
		);

		$response = parent::exec();

		$dataBase->close();

		if ($this->pageInfo && is_array($response->body)) {
			$paged = new \Models\PagedCollection;

			$paged->PageSize = $this->pageInfo->getPageSize();
			$paged->Page = $this->pageInfo->getPage();
			$paged->TotalPages = $this->pageInfo->getTotalPages();
			$paged->TotalItems = $this->pageInfo->getTotalItems();
			$paged->Items = $response->body;

			$response->body = $paged;
		}

		if (!is_null($response->body)) {
			$response->contentType = 'application/json; charset=utf-8';
			$response->body = json_encode($response->body);
		}

		return $response;
	}
}
