<?php
namespace Repositories {

	use Framework\Repository;
	use Framework\PageInfo;
	use Framework\SortInfo;
	use Framework\Filter;

	use Models\AlertProfile;

	class AlertProfileRepository extends Repository {

		private static $baseQuery = 'SELECT id, name, description, notify, notifyEmail FROM AlarmProfiles WHERE TRUE';

		public function getById($itemId) {
			$query = self::$baseQuery . ' AND id = ?';

			$statement = $this->dataBase->prepare();
			$statement->bind_param('i', $itemId);
			$statement->execute();

			$alertProfiles = $this->getItemsFromStatement($statement);

			$statement->close();

			if (count($alertProfiles) != 1) {
				return null;
			}

			return $alertProfiles[0];
		}

		public function getList(Filter $filter = null, PageInfo $pageInfo = null, SortInfo $sortInfo = null) {
			$query = self::$baseQuery;

			$statement = $this->dataBase->prepare($query);
			$statement->execute();

			$alertProfiles = $this->getItemsFromStatement($statement);

			$statement->close();

			return $alertProfiles;
		}

		protected function getItemsFromStatement(\mysqli_stmt $statement) {
			$alertProfiles = array();

			$statement->bind_result(
				$profileId,
				$profileName,
				$profileDescription,
				$profileNotify,
				$profileNotifyEmail
				);

			while ($statement->fetch()) {
				$profile = new AlertProfile;

				$profile->Id = $profileId;
				$profile->Name = $profileName;
				$profile->Description = $profileDescription;
				$profile->Notify = (boolean) $profileNotify;
				$profile->NotifyEmail = $profileNotifyEmail;

				$alertProfiles[] = clone $profile;
			}

			return $alertProfiles;
		}
	}
}
?>
