<?php

namespace Repositories {

	use \Framework\Repository;
	use \Framework\PageInfo;
	use \Framework\SortInfo;
	use \Framework\Filter;

	use \Models\DeviceType;

	class DeviceTypeRepository extends Repository {

		private static $baseQuery = 'SELECT id, name, mfg, class, defaultFirmwareId, defaultConfigId, defaultStatsProfileId, defaultAlarmProfileId FROM DeviceTypes WHERE TRUE';

		public function getById($itemId) {
			$query = self::$baseQuery . ' AND id = ?';

			$statement = $this->dataBase->prepare($query);
			$statement->bind_param('i', $itemId);
			$statement->execute();

			$deviceTypes = $this->getItemsFromStatement($statement);

			$statement->close();

			if (count($deviceTypes) != 1) {
				return null;
			}

			return $deviceTypes[0];
		}

		public function getList(Filter $filter = null, PageInfo $pageInfo = null, SortInfo $sortInfo = null) {
			$query = self::$baseQuery;

			$statement = $this->dataBase->prepare($query);
			$statement->execute();

			$deviceTypes = $this->getItemsFromStatement($statement);

			$statement->close();

			return $deviceTypes;
		}

		public function getItemsFromStatement(\mysqli_stmt $statement) {
			$statement->bind_result(
				$typeId,
				$typeName,
				$typeManufacturer,
				$typeClass,
				$typeDefaultFirmwareId,
				$typeDefaultConfigId,
				$typeDefaultStatsProfileId,
				$typeDefaultAlertProfileId
				);

			$deviceTypes = array();

			while ($statement->fetch()) {
				$type = new DeviceType;

				$type->Id = $typeId;
				$type->Name = $typeName;
				$type->Manufacturer = $typeManufacturer;
				$type->Class = $typeClass;
				$type->DefaultFirmwareId = $typeDefaultFirmwareId;
				$type->DefaultConfigId = $typeDefaultConfigId;
				$type->DefaultStatsProfileId = $typeDefaultStatsProfileId;
				$type->DefaultAlertProfileId = $typeDefaultAlertProfileId;

				$deviceTypes[] = $type;
			}

			return $deviceTypes;
		}
	}
}

?>
