<?php

namespace Repositories {

	use Framework\Repository;
	use Framework\PageInfo;
	use Framework\SortInfo;
	use Framework\Filter;
	
	use Models\NetworkInterface;
	
	class InterfaceRepository extends Repository {
	
		private static $columnsArray = array (
			'HostInterfaces.id',
			'HostInterfaces.hostId',
			'HostInterfaces.name',
			'HostInterfaces.type',
			'HostInterfaces.mac',
			'HostInterfaces.rssi'
		);
		
		private static function queryColumns() {
			return implode(', ', self::$columnsArray);
		}
		
		private static $tablesArray = array (
			'HostInterfaces',
			'INNER JOIN Hosts ON Hosts.id = HostInterfaces.hostId',
			'LEFT JOIN Networks ON Networks.id = Hosts.networkId'
		);
		
		private static function queryTables() {
			return implode(' ', self::$tablesArray);
		}
		
		private function baseQuery() {
			return $this->composeQuery(self::queryColumns(), self::queryTables(), 'NOT HostInterfaces.forDelete');
		}
	
		public function getById($itemId) {
			$query = $this->baseQuery() . ' AND HostInterfaces.hostId = ?';
			
			$statement = $this->dataBase->prepare($query);
			$statement->bind_param('i', $itemId);
			
			$statement->execute();
			
			$interfaces = $this->getItemsFromStatement($statement);
			
			$statement->close();
			
			$this->populatePeers($interfaces);
			
			return $interfaces;
		}
	
		public function getList(Filter $filter = null, PageInfo $pageInfo = null, SortInfo $sortInfo = null) {
			$query = $this->baseQuery();
			
			$statement = $this->dataBase->prepare($query);
			$statement->execute();
			
			$interfaces = $this->getItemsFromStatement($statement);
			
			$statement->close();
			
			$this->populatePeers($interfaces);
			
			return $interfaces;
		}
		
		protected function getItemsFromStatement(\mysqli_stmt $statement) {
			$interfaces = array();
			
			$statement->bind_result(
				$interfaceId,
				$interfaceInventoryId,
				$interfaceName,
				$interfaceType,
				$interfaceMacAddress,
				$interfaceRssi
			);
			
			$statement->execute();
			
			while ($statement->fetch()) {
				$interface = new NetworkInterface;
				
				$interface->Id = (string) $interfaceId;
				$interface->InventoryId = (string) $interfaceInventoryId;
				$interface->Name = $interfaceName;
				$interface->Type = $interfaceType;
				$interface->MacAddress = $interfaceMacAddress;
				$interface->Rssi = $interfaceRssi;
				
				$interfaces[] = $interface;
			}
			
			return $interfaces;
		}
		
		private function populatePeers($interfaces) {
			$statement = $this->dataBase->prepare('SELECT mac FROM HostInterfacePeers WHERE hostInterfaceId = ?');
			
			foreach ($interfaces as $interface) {
				$statement->bind_param('i', $interface->Id);
				$statement->execute();
				
				$interface->Peers = array();
				
				$statement->bind_result($peerMacAddress);
				
				while ($statement->fetch()) {
					$interface->Peers[] = $peerMacAddress;
				}
			}
			
			$statement->close();
		}
	}
}

?>
