<?php

namespace Repositories {
	use \Framework\Repository;
	
	use \Framework\Filter;
	use \Framework\PageInfo;
	use \Framework\SortInfo;

	use \Models\Network;

	class NetworkRepository extends Repository {
		private function getBaseQuery() {
			$query = 'SELECT id, parentid, name, ip, NetworkActiveAlarmCount(id) FROM Networks';

			if ($this->restrictedUser) {
				$query .= ', User2Network WHERE id = networkId';
			} else {
				$query .= ' WHERE TRUE';
			}

			$query .= ' ORDER BY Networks.name';

			return $query;
		}

		public function getById($itemId) {
			$query = $this->getBaseQuery() . ' AND id = ?';

			$statement = $this->dataBase->prepare($query);
			$statement->bind_param('i', $itemId);
			$statement->execute();

			$networks = $this->getItemsFromStatement($statement);

			$statement->close();

			if (count($networks) != 1) {
				return null;
			}

			return $networks[0];
		}

		public function getList(Filter $filter = null, PageInfo $page = null, SortInfo $sort = null) {
			$query = $this->getBaseQuery();

			$statement = $this->dataBase->prepare($query);
			$statement->execute();

			$networks = $this->getItemsFromStatement($statement);

			$statement->close();

			return $networks;
		}

		protected function getItemsFromStatement(\mysqli_stmt $statement) {
			$statement->bind_result(
				$networkId,
				$parentId,
				$name,
				$ipAddress,
				$alarmCount
			);

			$networks = array();

			while ($statement->fetch()) {
				$network = new Network;

				if (is_string($ipAddress) && strlen($ipAddress) == 0) {
					$ipAddress = null;
				}

				$network->Id = (string) $networkId;
				$network->ParentId = (string) $parentId;
				$network->Name = $name;
				$network->IPAddress = $ipAddress;
				$network->ActiveAlertCount = $alarmCount;

				$networks[] = $network;
			}

			return $networks;
		}
	}
}

?>
