<?php

namespace Repositories {

	use Framework\Repository;
	use Framework\PageInfo;
	use Framework\SortInfo;
	use Framework\Filter;
	
	use Models\StatsProfile;

	class StatsProfileRepository extends Repository {
	
		private static $selectStatement = 'SELECT id, name FROM StatsProfiles WHERE TRUE';
		private static $orderByStatement = ' ORDER BY ordinal';
	
		public function getById($itemId) {
			$query = self::$selectStatement . ' WHERE id = ?';
			
			$statement = $this->dataBase->prepare($query);
			$statement->bind_param('i', $itemId);
			$statement->execute();
			
			$profiles = $this->getItemsFromStatement($statement);
			
			$statement->close();
			
			if (count($profiles) != 1) {
				return null;
			}
			
			return $profiles[0];
		}
		
		public function getList(Filter $filter = null, PageInfo $pageInfo = null, SortInfo $sortInfo = null) {
			$query = self::$selectStatement . self::$orderByStatement;
			
			$statement = $this->dataBase->prepare($query);
			$statement->execute();
			
			$profiles = $this->getItemsFromStatement($statement);
			
			$statement->close();
			
			return $profiles;
		}
	
		protected function getItemsFromStatement(\mysqli_stmt $statement) {
			$profiles = array();
			
			$statement->bind_result(
				$profileId,
				$profileName
			);
			
			while ($statement->fetch()) {
				$profile = new StatsProfile;
				$profile->Id = $profileId;
				$profile->Name = $profileName;
				
				$profiles[] = $profile;
			}
			
			return $profiles;
		}
	}

}

?>
