<?php

namespace Repositories {

	use \Framework\Repository;
	use \Framework\Filter;
	use \Framework\PageInfo;
	use \Framework\SortInfo;

	use \Models\User;

	class UserRepository extends Repository {

		private static $baseQuery = 'SELECT Users.id, Users.username, UserGroups.`key`, CONCAT_WS(\' \', Users.firstname, Users.lastname), Users.email FROM Users LEFT JOIN UserGroups ON Users.userGroupId = UserGroups.id WHERE TRUE';


		public function getById($itemId) {
			$query = self::$baseQuery . ' AND Users.id = ?';

			$statement = $this->dataBase->prepare($query);
			$statement->bind_param('i', $itemId);
			$statement->execute();

			$users = $this->getItemsFromStatement($statement);

			if (count($users) != 1) {
				return NULL;
			}

			return $users[0];
		}

		public function getList(Filter $filter = null, PageInfo $page = null, SortInfo $sort = null) {
			$query = self::$baseQuery;

			$statement = $this->dataBase->prepare($query);
			$statement->execute();

			$users = $this->getItemsFromStatement($statement);

			$statement->close();

			return $users;
		}

		protected function getItemsFromStatement(\mysqli_stmt $statement) {
			$items = array();

			$statement->bind_result(
				$userId,
				$username,
				$userGroup,
				$userFullName,
				$userEmail
			);

			while ($statement->fetch()) {
				$user = new User;

				$user->Id = (string) $userId;
				$user->Username = $username;
				$user->Group = $userGroup;
				$user->FullName = $userFullName;
				$user->Email = $userEmail;

				$items[] = $user;
			}

			return $items;
		}
	}
}

?>
