<?php

use Framework\Filter;

class FilteredAlertsResource extends Framework\Resource {
	protected function filterArgs() {
		return array(
			'DateFrom'			=> Filter::RangeFrom,
			'DateTo'			=> Filter::RangeTo,
			'NameContains'			=> Filter::Contains,
			'MessageContains'		=> Filter::Contains,
			'InventoryNameContains'		=> Filter::Contains,
			'InventoryIPAddressContains'	=> Filter::Contains,
			'InventoryNetworkNameContains'	=> Filter::Contains,
			'TodoUserFullNameContains'	=> Filter::Contains,
			'InventoryNetworkId'		=> Filter::Equals,
			'Status'			=> Filter::Equals,
			'Severity'			=> Filter::Equals,
			'SeverityOneOf'			=> Filter::OneOf,
			'SearchString'			=> Filter::SearchString
		);
	}
	
	protected function searchCols() {
		return array('Name','Message', 'InventoryName', 'InventoryIPAddress', 'InventoryNetworkName', 'TodoUserFullName');
	}
}

/**
 * @uri /alerts
 */
class AlertsCollection extends FilteredAlertsResource {
	protected function sortCols() {
		return array('DateTime', 'Name', 'InventoryName', 'Severity', 'Status');
	}
	
	protected function defaultSortBy() {
		return 'DateTime,Severity';
	}
	
	protected function defaultSortOrder() {
		return 'D,D';
	}

	/**
	 * @method GET
	 * @provides application/json
	 */
	public function getList() {
		$repository = new Repositories\AlertRepository($this->dataBase,
			$this->restrictedUser,
			$this->userId
			);
		
		$alerts = $repository->getList(
			$this->filter,
			$this->pageInfo,
			$this->sortInfo
		);

		return new Tonic\Response(200, $alerts);
	}
}


/**
 * @uri /alerts/count
 */
class AlertsCountResource extends FilteredAlertsResource {
	/**
	 * @method GET
	 * @provides application/json
	 */
	public function getCount() {
		$repository = new Repositories\AlertRepository($this->dataBase,
			$this->restrictedUser,
			$this->userId
			);
		$groupBy = isset($_GET['GroupBy']) ? $_GET['GroupBy'] : null;
		
		$count = $repository->getCount($this->filter, $groupBy);
		
		if ($count == null) {
			return new Tonic\Response(400);
		}
		
		return new Tonic\Response(200, $count);
	}
}

/**
 * @uri /alerts/{id}
 */
class AlertResource extends Framework\Resource {
	/**
	 * @method GET
	 * @provides application/json
	 */
	public function getResorce($id) {
		$repository = new Repositories\AlertRepository($this->dataBase,
			$this->restrictedUser,
			$this->userId
			);

		$alert = $repository->getById($id);

		if ($alert == null) {
			throw new Tonic\NotFoundException;
		}

		return new Tonic\Response(200, $alert);
	}
}

?>
