<?php

use Framework\Filter;

class FilteredInventoryResource extends Framework\Resource {

	protected function filterArgs() {
		return array(
			'NameContains'		=> Filter::Contains,
			'IPAddressContains'	=> Filter::Contains,
			'NetworkNameContains'	=> Filter::Contains,
			'DeviceTypeOneOf'	=> Filter::OneOf,
			'NetworkId'		=> Filter::Equals,
			'IsOffline'		=> Filter::Equals,
			'IsRegistered'		=> Filter::Equals,
			'SearchString'		=> Filter::SearchString
		);
	}
	
	protected function searchCols() {
		return array('Name', 'IPAddress', 'NetworkName', 'DeviceType', 'Location', 'MacAddress', 'Firmware', 'SerialNumber');
	}
	
}

/**
 * @uri /inventory
 */
 
class InventoryCollection extends FilteredInventoryResource {
	protected function sortCols() {
		return array('IsOffline', 'IsRegistered', 'Name', 'IPAddress', 'NetworkName', 'DeviceType');
	}
	
	protected function defaultSortBy() {
		return 'IsRegistered,IsOffline,Name';
	}
	
	protected function defaultSortOrder() {
		return 'D,A,A';
	}

	/**
	 * @method GET
	 * @provides application/json
	 */
	public function getList() {
		$repository = new Repositories\InventoryRepository($this->dataBase,
			$this->restrictedUser,
			$this->userId
			);
		$inventory = $repository->getList(
			$this->filter,
			$this->pageInfo,
			$this->sortInfo
		);
		$response = new Tonic\Response(200, $inventory);
		return $response;
	}
	
}

/**
 * @uri /inventory/count
 */
class InventoryCountResource extends FilteredInventoryResource {
	/**
	 * @method GET
	 * @provides application/json
	 */
	public function getCount() {
		$repository = new Repositories\InventoryRepository($this->dataBase,
			$this->restrictedUser,
			$this->userId
			);
		$groupBy = isset($_GET['GroupBy']) ? $_GET['GroupBy'] : null;
		
		$count = $repository->getCount($this->filter, $groupBy);
		
		return $count == null
			? new Tonic\Response(400)
			: new Tonic\Response(200, $count);
	}
	
}

/**
 * @uri /inventory/{id}
 */

class InventoryResource extends Framework\Resource {
	/**
	 * @method GET
	 * @provides application/json
	 */
	public function getItem($id) {
		$repository = new Repositories\InventoryRepository($this->dataBase,
			$this->restrictedUser,
			$this->userId
			);
		$inventory = $repository->getById($id);
		if ($inventory == null) {
			throw new Tonic\NotFoundException;
		}
		$response = new Tonic\Response(200, $inventory);
		return $response;
	}
	
	/**
	 * @method DELETE
	 */
	public function deleteItem($id) {
		$repository = new Repositories\InventoryRepository($this->dataBase);
		$repository->delete($id);
		return new Tonic\Response(200);
	}
	
	/**
	 * @method POST
	 * @accepts application/json
	 */
	public function updateItem($id) {
		$data = $this->request->data;
		
		$inventory = new Models\Inventory;
		
		$inventory->Id = $id;
		$inventory->NetworkId = $data->NetworkId;
		$inventory->AlertProfileId = $data->AlertProfileId;
		$inventory->StatsProfileId = $data->StatsProfileId;
		$inventory->DeviceTypeId = $data->DeviceTypeId;
		$inventory->HeartbeatInterval = $data->HeartbeatInterval;
		$inventory->IsRegistered = $data->IsRegistered;
		
		$inventory->Coordinates = new Models\Wgs84Coordinates;
		
		$inventory->Coordinates->Longitude = $data->Coordinates->Longitude;
		$inventory->Coordinates->Latitude = $data->Coordinates->Latitude;

		/* 
		 * This is just to check whether the Height property is there,
		 * because support for it was added before the mobile application
		 * implemented it.
		 */
		
		$vars = get_object_vars($data);
		
		if (array_key_exists('Height', $vars)) {
			$inventory->Height = $data->Height;
		} else {
			$inventory->Height = FALSE;
		}
		
		$repository = new Repositories\InventoryRepository($this->dataBase);
		$repository->update($inventory);

		return new Tonic\Response(200);
	}
	
}

?>
