<?php

  class BasicMappingModel extends Model {
    public $name;
    public $type;
    public $value;
  }
  
  class BasicMappingSubhandler extends Handler {
    private $repository;
    private $profileId;
    
    public function __construct($profileId) {
      $this->profileId = $profileId;
    }
    
    public function init() {
      $this->repository = new MappingProfileRepository($this->getDBConnection());
    }
    
    
    public function getBasicMappingList() {
      $mappings = $this->repository->listBasicMappings($this->profileId);
      
      $document = $this->getResponseDocument();
      $root = $this->createRoot();
      
      $container = $this->addChild($root, "Mappings");
      
      foreach ($mappings as $mapping) {
        $model = new BasicMappingModel();
        $model->name = $mapping->name;
        $model->type = $mapping->type;
        $model->value = $mapping->value;
        $element = Model::toXMLElement($model, $document, "Mapping");
        $element->setAttribute("id", $mapping->id);
        $container->appendChild($element);
      }
    }
    
    public function getBasicMapping(IntParam $id, $addIDAttr = false) {
      $mapping = $this->repository->findBasicMapping($this->profileId, $id->value);
      $document = $this->getResponseDocument();
      $root = $this->createRoot();
      
      $model = new BasicMappingModel();
      $model->name = $mapping->name;
      $model->type = $mapping->type;
      $model->value = $mapping->value;
      $element = Model::toXMLElement($model, $document, "Mapping");
      
      if ($addIDAttr) {
        $element->setAttribute("id", $id->value);
      }
      
      $root->appendChild($element);
    }
    
    public function postBasicMapping(BasicMappingModel $model) {
    
      try {
        $insertId = $this->repository->saveBasicMapping($this->profileId, NULL, $model);
      } catch (RepositoryError $e) {
        if ($e->getCode() === 1062) {
          throw new HandlerError(HTTPStatus::Conflict, $e->getMessage());
        }
        throw new HandlerError(HTTPStatus::InternalServerError, $e->getMessage());
      }
      
      if ($insertId != NULL) {
        $this->setStatus(HTTPStatus::Created);
        $this->getBasicMapping(new IntParam($insertId), true);
      } else {
        $this->setStatus(HTTPStatus::NotFound);
      }
    }
    
    public function putBasicMapping(IntParam $id, BasicMappingModel $model) {
      try {
        $updateId = $this->repository->saveBasicMapping($this->profileId, $id->value, $model);
      } catch (RepositoryError $e) {
        $status = ($e->getCode() === 1062)
          ? HTTPStatus::Conflict
          : HTTPStatus::InternalServerError;
        throw new HandlerError($status, $e->getMessage());
      }
      $this->getBasicMapping($id);
    }
    
    public function deleteBasicMapping(IntParam $id) {
      $this->repository->deleteBasicMapping($this->profileId, $id->value);
    }
  }
?>