<?php

class CustomerServices extends Model {
	public $warrantyStart;
	public $warrantyPeriod;
	public $warrantyExtension;
	public $nextDayReplacementStart;
	public $nextDayReplacementPeriod;
	
	private static $propertyTypes = array (
		"warrantyStart" => "DateParam?",
		"warrantyPeriod" => "IntParam?",
		"warrantyExtension" => "IntParam?",
		"nextDayReplacementStart" => "DateParam?",
		"nextDayReplacementPeriod" => "IntParam?"
	);
	
	public function getPropertyTypes() {
		return self::$propertyTypes;
	}
}

class DeviceHandler extends Handler {
	private function formatModel($model) {
		
		$document = $this->getResponseDocument();
		
		$services = $document->createElement("WarrantyAndServices");
		
		$this->addChild($services, "WarrantyStart", $model->warrantyStart);
		$this->addChild($services, "WarrantyPeriod", $model->warrantyPeriod);
		$this->addChild($services, "WarrantyExtension", $model->warrantyExtension);
		$this->addChild($services, "NextDayReplacementStart", $model->nextDayReplacementStart);
		$this->addChild($services, "NextDayReplacementPeriod", $model->nextDayReplacementPeriod);
		
		return $services;
	}
	
	
	public function getCustomerServices(IntParam $id) {

		$querySQL = <<<EOQ
SELECT
	type,
	DATE_FORMAT(startDate, '%Y-%m-%d') AS startDate,
	period,
	extension
FROM
	HostCustomerServicesView
WHERE
	hostId = $id
EOQ;

		try {
			$rows = $this->selectObjects($querySQL);
		} catch (EmptyResultSetError $e) {
			$this->setStatus(HTTPStatus::NotFound);
			throw $e;
		}
		
		$model = new CustomerServices();
		
		foreach ($rows as $row) {
			if ($row->type == 'warranty') {
				$model->warrantyStart = $row->startDate;
				$model->warrantyPeriod = $row->period;
				$model->warrantyExtension = $row->extension;
			} else if ($row->type == 'ndr') {
				$model->nextDayReplacementStart = $row->startDate;
				$model->nextDayReplacementPeriod = $row->period;
			}
		}

		$modelElement = $this->formatModel($model);
		
		$root = $this->createRoot();
		$root->appendChild($modelElement);
	}

	public function putCustomerServices(IntParam $id, CustomerServices $model) {
	
		$preparedSQL = <<<EOQ
INSERT INTO HostCustomerServices (
	hostId,
	typeId,
	startDate,
	period,
	extension
) SELECT
	?,
	id,
	?,
	?,
	?
FROM
	CustomerServiceTypes
WHERE
	name = ?
EOQ;

		$db = $this->getDBConnection();
		
		$statement = $db->prepare($preparedSQL);
		$statement->bind_param("isiis", $hostId, $startDate, $period, $extension, $type);
		
		$db->autocommit(FALSE);
		
		$this->query("DELETE FROM HostCustomerServices WHERE hostId=" . $id);
		$rowsDeleted = $db->affected_rows;
		
		$hostId = $id->value;
		
		$type = 'warranty';
		$startDate = $model->warrantyStart;
		$period = $model->warrantyPeriod;
		$extension = $model->warrantyExtension;
		
		$statement->execute();
		
		if ($model->nextDayReplacementStart) {
			$type = 'ndr';
			$startDate = $model->nextDayReplacementStart;
			$period = $model->nextDayReplacementPeriod;
			$extension = NULL;
			
			$statement->execute();
		}
		
		$db->commit();

		$statement->close();

		$modelElement = $this->formatModel($model);
		
		$root = $this->createRoot();
		$root->appendChild($modelElement);

		if ($rowsDeleted == 0) {
			self::setStatus(HTTPStatus::Created);
		}
	}
}

?>