<?php

  class MappingProfileModel extends Model {
    public $name;
  }
  
  class MappingProfileHandler extends Handler {
  
    private $repository;
    
    public function init() {
      $this->repository = new MappingProfileRepository($this->getDBConnection());
    }
  
    public static function basicMappingHandler(IntParam $mappingProfileId) {
      return new BasicMappingSubhandler($mappingProfileId->value);
    }
    
    //public static function customMappingHandler(IntParam $mappingProfileId) {
    //  return new CustomMappingSubhandler($mappingProfileId->value);
    //}
    
    public function getMappingProfileList() {
      $profiles = $this->repository->listProfiles();
      
      $document = $this->getResponseDocument();
      $root = $this->createRoot();
      $container = $this->addChild($root, "profiles");
      
      foreach ($profiles as $profile) {
        $model = new MappingProfileModel();
        $model->name = $profile->name;
        $element = Model::toXMLElement($model, $document, "profile");
        $element->setAttribute("id", $profile->id);
        $container->appendChild($element);
      }
    }
    
    public function postMappingProfile(MappingProfileModel $model) {
      try {
        $id = $this->repository->saveProfile(NULL, $model);
      } catch (RepositoryError $e) {
        $status = $e->getCode() == 1062
          ? HTTPStatus::Conflict
          : HTTPStatus::InternalServerErrror;
        throw new HandlerError($status, $e->getMessage());
      }
      
      $this->setStatus(HTTPStatus::Created);
      $this->getMappingProfile(new IntParam($id), true);
    }
    
    public function putMappingProfile(IntParam $id, MappingProfileModel $model) {
      try {
        $insertId = $this->repository->saveProfile($id->value, $model);
      } catch (RepositoryError $e) {
        $status = $e->getCode() == 1062
          ? HTTPStatus::Conflict
          : HTTPStatus::InternalServerErrror;
        throw new HandlerError($status, $e->getMessage());
      }
      
      if ($insertId === NULL) {
        $this->setStatus(HTTPStatus::NotFound);
        return;
      }
      
      $this->getMappingProfile($id);
    }
    
    public function getMappingProfile(IntParam $id, $addIDAttr = false) {
      $profile = $this->repository->findProfile($id->value);
      
      if ($profile === null) {
        throw new HandlerError(HTTPStatus::NotFound, "No such profile");
      }
      
      $model = new MappingProfileModel();
      $model->name = $profile->name;
      
      $root = $this->createRoot();
      $element = Model::toXMLElement($model,
        $this->getResponseDocument(),
        "profile"
        );
      if ($addIDAttr) {
        $element->setAttribute("id", $profile->id);
      }
      $root->appendChild($element);
    
    }
    
    public function deleteMappingProfile(IntParam $id) {
      $this->repository->deleteProfile($id->value);
    }
  }

?>