<?php

  class MappingProfileRepository extends Repository {
    public function __construct(MySQLi $dbConnection) {
      parent::__construct($dbConnection);
    }
    
    public function listProfiles() {
      return $this->fetchObjects(MappingProfileRepository::$listProfilesQuery);
    }
    
    public function findProfile($id) {
      $query = MappingProfileRepository::$findProfileQuery;
      $query = str_replace("?", $id, $query);
      return $this->fetchObject($query);
    }
    
    public function saveProfile($id, MappingProfileModel $model) {
      if ($id === null) {
        $statement = $this->prepare(MappingProfileRepository::$insertProfileQuery);
        $statement->bind_param("s", $model->name);
      } else {
        $statement = $this->prepare(MappingProfileRepository::$updateProfileQuery);
        $statement->bind_param("si", $model->name, $id);
      }
      
      $statement->execute();
      
      $errno = $statement->errno;
      $error = $statement->error;
      
      $affectedRows = $statement->affected_rows;
      $insertId = $statement->insert_id;
      
      $statement->close();

      if ($errno !== 0) {
        throw new RepositoryError($errno, $error);
      }
      
      $profileId = $id === NULL ? $insertId : $id;
      
      return $profileId;
    }
    
    public function deleteProfile($id) {
      $statement = $this->prepare(MappingProfileRepository::$deleteProfileQuery);
      $statement->bind_param("i", $id);
      $statement->execute();
      $statement->close();
    }
    
    public function listBasicMappings($profileId) {
      $query = MappingProfileRepository::$listBasicMappingsQuery;
      $query = str_replace("?", $profileId, $query);
      
      $result = $this->query($query);
      
      $mappings = array();
      
      while ($mapping = $result->fetch_object()) {
        $mappings[] = $mapping;
      }
      
      $result->free();
      
      return $mappings;
    }
    
    public function findBasicMapping($profileId, $mappingId) {
      $statement = $this->prepare(MappingProfileRepository::$findBasicMappingQuery);
      $statement->bind_param("ii", $mappingId, $profileId);
      $statement->execute();
      $statement->bind_result($mapping->name, $mapping->type, $mapping->value);
      $statement->fetch();
      $statement->close();
      
      return $mapping;
    }
    
    public function saveBasicMapping($profileId, $mappingId, BasicMappingModel $model) {
      if ($mappingId === NULL) {
        $statement = $this->prepare(MappingProfileRepository::$insertBasicMappingQuery);
        $statement->bind_param("isss", $profileId, $model->name, $model->type, $model->value);
      } else {
        $statement = $this->prepare(MappingProfileRepository::$updateBasicMappingQuery);
        $statement->bind_param("sssii", $model->name, $model->type, $model->value, $mappingId, $profileId);
      }
      
      $statement->execute();
      
      if ($statement->errno !== 0) {
        $errno = $statement->errno;
        $error = $statement->error;
        $statement->close();
        throw new RepositoryError($errno, $error);
      }
      
      
      $insertId = $statement->insert_id;
      $affectedRows = $statement->affected_rows;
      
      $statement->close();
      
      return $affectedRows < 1 ? NULL : ($mappingId === NULL ? $insertId : $mappingId);
    }
    
    public function deleteBasicMapping($profileId, $mappingId) {
      //$statement = $this->prepare(BasicMappingQuery::DELETE);
      $statement = $this->prepare(MappingProfileRepository::$deleteBasicMappingQuery);
      $statement->bind_param("ii", $mappingId, $profileId);
      $statement->execute();
      $statement->close();
    }
    
    private static $listProfilesQuery = "SELECT id, name FROM PropertyMappingProfiles ORDER BY id";
    private static $findProfileQuery = "SELECT id, name FROM PropertyMappingProfiles WHERE id = ?";

    private static $insertProfileQuery = <<<EOQ
INSERT INTO PropertyMappingProfiles (name) VALUES (?)
EOQ;

    private static $updateProfileQuery = <<<EOQ
UPDATE
  PropertyMappingProfiles
SET
  name = ?
WHERE
  id = ?
EOQ;

    private static $deleteProfileQuery = "DELETE FROM PropertyMappingProfiles WHERE id = ?";

    private static $listBasicMappingsQuery = <<<EOQ
SELECT
  id,
  name,
  type,
  value
FROM
  BasicPropertyMappings
WHERE
  profileId = ?
ORDER BY
  id
EOQ;

  private static $findBasicMappingQuery = <<<EOQ
SELECT
  name,
  type,
  value
FROM
  BasicPropertyMappings
WHERE
  id = ?
  AND profileId = ?
EOQ;

  private static $insertBasicMappingQuery = <<<EOQ
INSERT INTO BasicPropertyMappings(
  profileId,
  name,
  type,
  value
) VALUES (
  ?,
  ?,
  ?,
  ?
)
EOQ;

  private static $updateBasicMappingQuery = <<<EOQ
UPDATE
  BasicPropertyMappings
SET
  name = ?,
  type = ?,
  value = ?
WHERE
  id = ? AND profileId = ?
EOQ;

  private static $deleteBasicMappingQuery = <<<EOQ
DELETE FROM
  BasicPropertyMappings
WHERE
  id = ? AND profileId = ?
EOQ;
  }
?>