<?php

class SweapDeviceModel extends Model {
  public $ipAddress;
  public $snmpVersion;
  public $snmpCommunity;
  public $mappingProfileId;
}

class SweapDeviceSubhandler extends Handler {
  public function __construct(IntParam $sweapId) {
    $this->sweapId = $sweapId->value;
  }
  
  private function queryDevices(IntParam $byId = null) {
    $idCondition = ($byId === null) ? "" : "AND id = {$byId->value}";
  
    $querySQL = <<<EOQ
SELECT
  id,
  hostId,
  sweapId,
  INET_NTOA(ipAddress) AS ipAddress,
  snmpVersion,
  snmpCommunity,
  mappingProfileId
FROM
  SweapDevices
WHERE
  sweapId = {$this->sweapId}
  {$idCondition}
EOQ;

    try {
      return ($byId === null)
        ? $this->selectObjects($querySQL)
        : $this->selectObject($querySQL);
    } catch (EmptyResultSetError $e) {
      if ($byId === null) {
        return array();
      }
    
      $this->setStatus(HTTPStatus::NotFound);
      throw $e;
    }
  }
  
  public function getDeviceList() {
    $rows = $this->queryDevices();
  
    $root = $this->createRoot();
    $devices = $this->addChild($root, "devices");
    
    $document = $this->getResponseDocument();
    
    foreach ($rows as $row) {
      $model = new SweapDeviceModel();
      $model->copyProperties($row);
      $element = $model->toXML($document, "device");
      $element->setAttribute("id", $row->id);
      if ($row->hostId !== NULL) {
        $element->setAttribute("hostId", $row->hostId);
      }
      $devices->appendChild($element);
    }
  }
  
  public function getDevice(IntParam $id, $addIDAttr = false) {
    $device = $this->queryDevices($id);
    
    $root = $this->createRoot();
    
    $model = new SweapDeviceModel();
    $model->copyProperties($device);
    
    $element = $model->toXML($this->getResponseDocument(), "device");
    
    if ($addIDAttr) {
      $element->setAttribute("id", $device->id);
    }
    
    if ($device->hostId !== NULL) {
      $element->setAttribute("hostId", $device->hostId);
    }
      
    $root->appendChild($element);
  }
  
  public function postDevice(SweapDeviceModel $model) {
    
    $insertSQL = <<<EOQ
INSERT INTO SweapDevices (
  sweapId,
  ipAddress,
  snmpVersion,
  snmpCommunity,
  mappingProfileId
) VALUES (
  ?,
  INET_ATON(?),
  ?,
  ?,
  ?
)
EOQ;
    $db = $this->getDBConnection();
    
    $statement = $db->prepare($insertSQL);
    $statement->bind_param("isssi", 
      $this->sweapId, 
      $model->ipAddress, 
      $model->snmpVersion, 
      $model->snmpCommunity, 
      $model->mappingProfileId
      );
    $statement->execute();
    
    $rowsInserted = $statement->affected_rows;
    $error = $statement->error;
    $id = $statement->insert_id;

    $statement->close();
    
    if ($rowsInserted < 1) {
      throw new HandlerError(HTTPStatus::BadRequest, $error);
    } else {
      $this->setStatus(HTTPStatus::Created);
      $this->getDevice(new IntParam($id), true);
    }
  }
  
  public function deleteDevice(IntParam $id) {
    $querySQL = "DELETE FROM SweapDevices WHERE id = {$id->value} AND sweapId = {$this->sweapId}";
    $this->query($querySQL);
  }
  
  public function putDevice(IntParam $id, SweapDeviceModel $model) {
    $updateSQL = <<<EOQ
UPDATE
  SweapDevices
SET
  ipAddress = INET_ATON(?),
  snmpVersion = ?,
  snmpCommunity = ?,
  mappingProfileId = ?
WHERE
  id = ?
EOQ;

    $db = $this->getDBConnection();

   $statement = $db->prepare($updateSQL);
   $statement->bind_param("sssii",
      $model->ipAddress,
      $model->snmpVersion,
      $model->snmpCommunity,
      $model->mappingProfileId,
      $id->value
      );
    $statement->execute();
    
    if ($statement->affected_rows < 1 && $statement->errno !== 0) {
      $error = $statement->error;
      $statement->close();
      throw new HandlerError(HTTPStatus::BadRequest, $error);
    }
    
    $statement->close();
    
    $this->getDevice($id);
  }
  
  private $sweapId;
}
?>