<?php

//
// SystemHandler -- a handler class for the /Backend/System URI.
// Implements various system-related services and provides access to
// system resources like settings, supported languages etc.
//

class SystemHandler extends Handler {

	//
	// Utility method that fetches all settings from the database
	//
	
	private function fetchSettings() {
		$settings = array();

		$result = $this->query("SELECT `key`, value FROM SystemSettings");

		while ($row = $result->fetch_row()) {
			$settings[$row[0]] = $row[1];
		}

		$result->free();
		
		return $settings;
	}
	
	//
	// Utilty function that fetches a single named settings from the
	// database
	//
	
	private function fetchSetting($name) {
		return $this->selectSingle("SELECT value FROM SystemSettings WHERE `key`='"
			. $this->escapeString($name)
			. "'");
	}
	
	//
	// Utility method that retrieves the current language setting
	// from the database as a locale name.
	//
	
	private function fetchCurrentLanguage() {
		return $this->selectSingle("SELECT locale FROM Languages, SystemSettings WHERE `key` = 'SystemLanguageId' AND Languages.id = value");
	}
	
	//
	// GET method for the /System/Settings resource collection.
	// Authorization: required
	// Parameters: none
	// Returns: list of settings from the SystemSettings table
	//

	public function getSettings() {
		
		$settings = $this->fetchSettings();
		
		$root = $this->createRoot();
		$collection = $this->addChild($root, "SystemSettings");
		
		foreach ($settings as $name => $value) {
			$setting = $this->addChild($collection, "Setting");
			$this->addChild($setting, "Name", $name);
			$this->addChild($setting, "Value", $value);
		}
	}
	
	//
	// GET method for the /System/SchemaVersion resource
	// Authorization: not required
	// Parameters: none
	// Returns: the DB schema version as in the 'SchemaVersion' record 
	//   in SystemSettings table
	//
	
	public function noauthGetSchemaVersion() {
		$schemaVersion = $this->fetchSetting("SchemaVersion");
		$root = $this->createRoot();
		$this->addChild($root, "SchemaVersion", $schemaVersion);
	}
	
	//
	// GET method for the /System/CurrentLanguage resource
	// Authorization: not required
	// Parameters: none
	// Returns: locale name of the current language used by the
	//   system, as (indirectly) specified by the 'SystemLanguageId'
	//   record in SystemSettings table
	//
	
	public function noauthGetCurrentLanguage() {
		$localeName = $this->fetchCurrentLanguage();
		$root = $this->createRoot();
		$this->addChild($root, "CurrentLanguage", $localeName);
	}
	
	//
	// PUT method for the /System/CurrentLanguage resource
	// Authorization: required
	// Parameters: locale - locale name that identifies the language
	//   to make current (as returned by the /System/Languages
	//   collection or /System/CurrentLanguage resource)
	// Returns: the effective active system language locale name
	//   after the operation (value different from the parameter
	//   passed indicates that an error has occured and language
	//   change did not happen)
	//
	
	public function putCurrentLanguage($locale) {
		$effectiveLocale = $this->selectSingle("SELECT change_system_language('"
			. $this->escapeString($locale)
			. "')");
		$root = $this->createRoot();
		$this->addChild($root, "CurrentLanguage", $effectiveLocale);
	}

	//
	// GET method for the /System/Languages resource collection
	// Authorization: required
	// Parameters: none
	// Returns: a list of languages supported by the system (from the
	//   Languages table) and the currently active system language ID
	//
	
	public function getLanguages() {
		$result = $this->query("SELECT locale, name FROM Languages WHERE enabled=1 ORDER BY name");
		
		$root = $this->createRoot();
		
		$current = $this->fetchCurrentLanguage();
		$this->addChild($root, "CurrentLanguage", $current);

		$collection = $this->addChild($root, "Languages");
		
		while ($row = $result->fetch_row()) {
			$language = $this->addChild($collection, "Language");
			$this->addChild($language, "Locale", $row[0]);
			$this->addChild($language, "Name", $row[1]);
		}
		
		$result->free();
	}
}

?>
