<?php

include('lib/sql.php');
include('lib/permissions.php');
include('lib/user.php');
include('lib/manage-alarm.php');

if (!function_exists('alarms_wrapper'))
{

	function alarms_wrapper()
	{
	}


	function handleAlarmAction($action)
	{
		if ($action == null)
			error('Unauthorized');

		switch ($action)
		{
			case 'search':
				echo tag('result', searchAlarms());

				break;
			// Only used for dashboard now
			case 'get':
				echo tag('result', getAlarms(req('pageNumber'), req('pageSize')));
				break;

			case 'clear':
				echo clearAlarms(req('alarmIds'));

				break;
				
			case 'remove':
				echo removeAlarms(req('alarmIds'));
				break;

			case 'clear-all':
				echo clearAllAlarms(req('hostId'));
				break;

			case 'clear-non-active':
				echo clearNonActive(req('hostId'));
				break;

			case 'ignore':
				echo ignoreAlarms(req('alarms'), req('endtime'));

				break;

			case 'get-ignore':
				echo tag('result', getIgnoredAlarms());

				break;

			case 'stop-ignore':
				echo stopIgnoringAlarm(req('alarmDefId'), req('hostId'));

				break;

		}

	}

	/*******************************************************************
	 * This is a combination of alarms:getAlarms and host:searchAlarms
	 *
	 */
	function searchAlarms()
	{

		global $currentUserId;

		$pageNumber = (int) req('pageNumber');
		$pageSize = (int) req('pageSize');
		$onlyActive = (int) req('onlyActive');

		$dateStart = cleanParam(req('dateStart'));
		$dateEnd = cleanParam(req('dateEnd'));

		$sortField = cleanParam(req('sortField'));
		$sortDirection = cleanParam(req('sortDirection'));

		$s_severities = req('severities');
		$s_hostNames = req('name'); // This is list of device names like rika-ralink%,%mimo%, etc...
		$s_alarmDefs = req('alarmDefs');
		$s_alarmMsg = req('alarmMsg');
		$s_networks = req('networks');
		$s_ip = req('ips'); // This is a list of ips to filter by
		
		$hostId = req('hostId');
		
		$dateFormat = sqlString("SELECT value FROM UserSettings, Users WHERE `key`='DateFormat' AND userId=Users.id AND guid='{$_SESSION['uguid']}'", "value");

		/* Process severities */
		$otherSeverities = sqlNumber("SELECT COUNT(*) AS otherSeverities FROM AlarmProfile2AlarmDef WHERE severity NOT REGEXP '^{$s_severities}$'", "otherSeverities");
		if ($otherSeverities == 0) {
			$s_severities = "";
		}

		$severities = explode('|', $s_severities);
		$severityQuery = "";
		foreach ($severities as $severity)
		{
			if (strlen($severity) < 2)
				continue;

			$severity = cleanParam($severity); // JIC
			if (strlen($severityQuery) > 0)
				$severityQuery .= " OR ";
			$severityQuery .= " AlarmProfile2AlarmDef.severity = '$severity' ";

		}

		/* Process host names */
		$hostNames = explode(',', $s_hostNames);
		$hostNameQuery = "";
		foreach ($hostNames as $name)
		{
			if (strlen($name) <= 0)
				continue;

			$name = cleanParam($name);
			$name = trim($name);
			if (strlen($hostNameQuery) > 0)
				$hostNameQuery .= " OR ";
			$hostNameQuery .= " Hosts.name like '$name' ";
		}

		/* Process alarm messages */
		$alarmMsg = explode(',', $s_alarmMsg);
		$alarmMsgQuery = "";
		foreach ($alarmMsg as $msg)
		{
			if (strlen($msg) <= 0)
				continue;

			$msg = cleanParam($msg);
			$msg = trim($msg);
			if (strlen($alarmMsgQuery) > 0)
				$alarmMsgQuery .= " OR ";
			$alarmMsgQuery .= " Alarms.msg like '$msg' ";
		}

		/* Process alarm defs */
		
		$otherDefs = sqlNumber("SELECT COUNT(*) AS otherDefs FROM AlarmProfile2AlarmDef WHERE alarmId NOT REGEXP '^{$s_alarmDefs}$'", "otherDefs");
		if ($otherDefs == 0) {
			$s_alarmDefs = "";
		}
		
		$alarmDefs = explode('|', $s_alarmDefs);
		$alarmDefQuery = "";
		foreach ($alarmDefs as $def)
		{
			if (strlen($def) <= 0)
				continue;

			$def = (int) $def;

			if (strlen($alarmDefQuery) > 0)
				$alarmDefQuery .= " OR ";
			$alarmDefQuery .= " Alarms.alarmid = $def ";
		}

		/* Process IP query */
		$hostIps = explode(',', $s_ip);
		$hostIpQuery = "";
		foreach ($hostIps as $ip)
		{
			if (strlen($ip) <= 0)
				continue;

			$ip = cleanParam($ip);
			$ip = str_replace(" ", "", $ip);

			if (strlen($hostIpQuery) > 0)
				$hostIpQuery .= " OR ";
			$hostIpQuery .= " INET_NTOA(Hosts.ipaddr) like '" . $ip . "' ";
		}
		
		
		$otherNetworks = sqlNumber("SELECT COUNT(*) AS otherNetworks FROM Networks WHERE id NOT IN ({$s_networks})", "otherNetworks");
		$searchAllNetworks = ($otherNetworks == 0);
		if ($searchAllNetworks) {
			$s_networks = "";
		}

		$networkids = explode(',', $s_networks);
		$isRestricted = isUserNetworkUser($currentUserId);

		// If don't specify networks, then use the user's list of allowed network
		if ($isRestricted && (count($networkids) == 0 || $s_networks == '')) {
			$networkids = getAllowableNetworkIds($currentUserId);
		}

		$page = ($pageNumber - 1) * $pageSize;

		$retVal = "";

		$recordCountQuery = " SELECT Count(Alarms.id) as Total ";

		$query = " SELECT "
				. " AlarmProfile2AlarmDef.severity, "
				. " AlarmDefinition.id as alarmDefId, "
				. " AlarmDefinition.name, "
				. " Alarms.id as alarmId, "
				. " Alarms.hostid, "
				. " Alarms.status, "
				. " Alarms.alarmid as alarmDefId, "
				. " Alarms.msg, "
				. " Alarms.time, "
				. " Hosts.serialno, "
				. " INET_NTOA(Hosts.ipaddr) as ip, "
				. " Hosts.name as deviceName, "
				. " HostCustomerServicesStatusSummaryView.statusSummary AS warrantyStatus, "
				. " Hosts.mac, "
				. " Networks.name as network ";

		$subquery =
				" FROM Alarms "
						. ($onlyActive ? " LEFT JOIN ActiveAlarm ON ActiveAlarm.alarm_id = Alarms.id " : "")
						. " , AlarmDefinition, AlarmProfile2AlarmDef, Hosts "
						. " LEFT JOIN HostCustomerServicesStatusSummaryView ON HostCustomerServicesStatusSummaryView.hostId = Hosts.id "
						. " LEFT JOIN Networks ON Networks.id = Hosts.networkId "
						. " WHERE 1 "
						. ($hostId != "" ? "" : " AND Alarms.cleared = 0 ")
						. (strlen($s_alarmDefs) > 0 ? " AND ($alarmDefQuery) " : "")
						. (strlen($s_hostNames) > 0 ? " AND ($hostNameQuery) " : "")
						. (strlen($s_ip) > 0 ? " AND ($hostIpQuery) " : "")
						. (strlen($s_severities) > 0 ? " AND ($severityQuery) " : "")
						. (strlen($s_alarmMsg) > 0 ? " AND ($alarmMsgQuery) " : "")
						. ($dateEnd != "" ? " AND Alarms.time <= '$dateEnd' " : "")
						. ($dateStart != "" ? " AND Alarms.time >= '$dateStart' " : "")
						. " AND Alarms.alarmid = AlarmDefinition.id "
						. " AND Alarms.hostid = Hosts.id "
						. " AND AlarmProfile2AlarmDef.alarmProfileId = Hosts.alarmProfileId "
						. " AND AlarmProfile2AlarmDef.alarmid = AlarmDefinition.id ";
						
		if ($hostId != "")
		{
			$subquery .= " AND Hosts.id = '$hostId' ";
		}

		$alertTypeQuery = "";

		// Then get alarms with status of 'notify' or alarms that are in the active alarms table
		if ($onlyActive)
		{
			$subquery .= " AND (ActiveAlarm.alarm_id is not null) ";

		}

		$query .= $subquery;
		$recordCountQuery .= $subquery;

		if (strlen($alertTypeQuery) > 0)
		{
			$query .= " AND ( $alertTypeQuery ) ";
			$recordCountQuery .= " AND ($alertTypeQuery) ";
		}

		$networkQuery = "";

		$searchNoNet = false;

		if (!$isRestricted && $searchAllNetworks)
		{
			$networkQuery .= " AND (1 ";
		}
		else if (count($networkids) > 0)
		{
			$networkQuery .= " AND (0 ";

			$i = 0;
			foreach ($networkids as $networkid)
			{

				$networkid = (int) $networkid;

				if ($networkid == 0)
				{
					$searchNoNet = true;
					continue;
				}

				if ($networkid > 0)
				{
					$networkQuery .= " OR  Networks.id = $networkid ";
				}
				$i++;
			}
		}
		else if (!$searchAllNetworks)
		{
			$searchNoNet = true;
		}

		if ($searchNoNet)
		{
			if (strlen($networkQuery) == 0)
			{
				$networkQuery = " AND (0 ";
			}

			$networkQuery .= " OR Networks.id = 0 OR Networks.id IS NULL)";
		}
		else if (strlen($networkQuery) > 0) 
		{
			$networkQuery .= ")";
		}

		/* 
		 * Whatever fuzzy logic has been applied to assemble the network
		 * lookup query, it still makes no sense when selecting alerts for
		 * a particular device.
		 */
		 
		 if ($hostId != "")
		 {
		   $networkQuery = "";
		 }

		$recordCountQuery .= $networkQuery;
		$query .= $networkQuery;

		// Cast alarm sort field, on non defined will get empty string
		if ($sortField != "")
		{
			$sortField = getSortByField_alarms($sortField);
		}

		// If we have empty, default to time
		if ($sortField == "")
		{
			$sortField = "Alarms.time";
			$sortDirection = "DESC";

		}


		$query .= " Order By  $sortField $sortDirection LIMIT $page, $pageSize ";

		$totalRecords = @sqlNumber($recordCountQuery, "Total");

		$conn = @getConnection();

		$result = mysql_query($query, $conn);

		if (!$result)
		{
			error("Unable to retrieve alarms from the database! " . mysql_error());
		}

		$numrows = mysql_numrows($result);

		$i = 0;
		$list = "";

		while ($i < $numrows)
		{
			$row = mysql_fetch_assoc($result);
			$id = (int) $row['alarmId'];
			$status = $row['status'];

			$list .= getStdAlarmDefDisplayName($row['name']);

			$retVal .= "<alarm id='" . $id . "' status='" . $status . "'>\n";

			$retVal .= tag('id', $id);
			$retVal .= tag('name', getStdAlarmDefDisplayName($row['name']), true);
			$retVal .= tag('severity', $row['severity']);
			$retVal .= tag('msg', $row['msg'], true);
			$retVal .= tag('time', $row['time'], true);
			$retVal .= tag('hostid', $row['hostid']);
			$retVal .= tag('ip', $row['ip']);
			$retVal .= tag('alarmDefId', $row['alarmDefId']);
			$retVal .= tag('serialno', $row['serialno'], true);
			$retVal .= tag('deviceName', $row['deviceName'], true);
			$retVal .= tag('mac', $row['mac'], true);
			
			$warrantyStatus = $row['warrantyStatus'];
			
			if ($dateFormat == 'us') {
				$warrantyStatus = preg_replace('/(20\d{2})-(\d{2})-(\d{2})/', '$2/$3/$1', $warrantyStatus);
			}
			
			$retVal .= tag('warranty', $warrantyStatus, true);
			
			$retVal .= tag('network', $row['network'], true);

			$retVal .= "</alarm>\n";
			$i++;
		}

		$retVal = tag('alarms', $retVal);
		$retVal .= tag('totalCount', $totalRecords);
		return $retVal;


	}

	/*********************************************************
	 * Get the current alarms for the system to show on the dashboard
	 *
	 */
	function getDashboardAlarms($maxAlerts, $alertTypes, $showClosed, $showNotify)
	{

		return getAlarms(1, $maxAlerts, $alertTypes, $showClosed, $showNotify);
	}

	/*********************************************************
	 * Get the current alarms for the system
	 *
	 */
	function getAlarms($pageNumber, $pageSize, $alertTypes = "", $showClosed = 0, $showNotify = 0)
	{
		/* If user is a network user, then we need to only get alarms for the
		*  devices in their networks. 	
		*/

		$showClosed = (int) $showClosed;
		$showNotify = (int) $showNotify;
		$pageNumber = (int) $pageNumber;
		$pageSize = (int) $pageSize;
		global $currentUserId;

		$isRestricted = isUserNetworkUser($currentUserId);
		$networkids = array();

		if ($isRestricted)
			$networkids = getAllowableNetworkIds($currentUserId);

		$page = ($pageNumber - 1) * $pageSize;

		$retVal = "";


		$recordCountQuery = " SELECT Count(Alarms.id) as Total ";

		$query = " SELECT "
				. " AlarmProfile2AlarmDef.severity, "
				. " AlarmDefinition.id as alarmDefId, "
				. " AlarmDefinition.name, "
				. " Alarms.id as alarmId, "
				. " Alarms.hostid, "
				. " Alarms.status, "
				. " Alarms.alarmid as alarmDefId, "
				. " HostCustomerServicesStatusSummaryView.statusSummary AS warrantyStatus, "
				. " Alarms.msg, "
				. " Alarms.time, "
				. " Hosts.serialno, "
				. " INET_NTOA(Hosts.ipaddr) as ip, "
				. " Hosts.name as deviceName, "
				. " Hosts.mac, "
				. " Networks.name as network ";

		$subquery = " FROM AlarmDefinition, AlarmProfile2AlarmDef, "
				. " Alarms "
				. ($showClosed == 0 ? " Left Join ActiveAlarm On ActiveAlarm.alarm_id = Alarms.id, " : " , ")
				. " Hosts "
				. " Left Join Networks "
				. " On Hosts.networkId = Networks.id "
				. " LEFT JOIN HostCustomerServicesStatusSummaryView ON HostCustomerServicesStatusSummaryView.hostId = Hosts.id "
				. " WHERE "
				. " Alarms.cleared = 0 "
				. ($showClosed == 0 ? " AND ( 0 OR ActiveAlarm.alarm_id is not null " . ($showNotify == 1 ? " OR Alarms.status like 'inactive') " : ")")
						: "")
				. " AND Alarms.hostid = Hosts.id "
				. ($showNotify == 0 ? " AND Alarms.status not like 'inactive' " : "")
				. " AND Alarms.alarmid = AlarmDefinition.id "
				. " AND AlarmProfile2AlarmDef.alarmProfileId = Hosts.alarmProfileId "
				. " AND AlarmProfile2AlarmDef.alarmid = AlarmDefinition.id "
				. "  ";

		$alertTypeQuery = "";

		if ($alertTypes != "")
		{

			foreach ($alertTypes as $alertType)
			{
				if ($alertType == "") continue;

				if (strlen($alertTypeQuery) > 0)
					$alertTypeQuery .= " OR ";

				$alertTypeQuery .= " AlarmProfile2AlarmDef.severity = '$alertType' ";
			}
		}

		$query .= $subquery;
		$recordCountQuery .= $subquery;

		// This is only used for the dashboard stats, so don't need to add to the
		// record count query since we dont care about the record count. Acually we do...
		if (strlen($alertTypeQuery) > 0)
		{
			$query .= " AND ( $alertTypeQuery ) ";
			$recordCountQuery .= " AND ($alertTypeQuery) ";
		}


		$networkQuery = "";

		if ($isRestricted && count($networkids) > 0)
		{
			$networkQuery .= " AND (0 ";

			$i = 0;
			foreach ($networkids as $networkid)
			{
				$networkQuery .= " OR ";

				$networkQuery .= " Networks.id = $networkid ";

				$i++;
			}
			$networkQuery .= ")";

		}

		$recordCountQuery .= $networkQuery;
		$query .= $networkQuery;

		$query .= " Order By Alarms.time Desc LIMIT $page, $pageSize ";

		/*$totalRecords = @sqlNumber($recordCountQuery, "Total");
		*/
		$conn = @getConnection();

		$result = @mysql_query($query, $conn);


		if (!$result)
			error("Unable to retrieve alarms from the database! " . mysql_error());

		$numrows = @mysql_numrows($result);

		$i = 0;

		$hasActive = 0;
		
		$list = "";

		while ($i < $numrows)
		{
			$row = mysql_fetch_assoc($result);
			$id = (int) $row['alarmId'];
			$status = $row['status'];

			if ($status != 'inactive')
				$hasActive = 1;

			$list .= getStdAlarmDefDisplayName($row['name']);

			$attr = " id='$id' status='$status' ";

			$retVal .= "<alarm $attr>\n";

			//$retVal .= tag('name', $row['name'], true); 
			$retVal .= tag('id', $id);
			$retVal .= tag('name', getStdAlarmDefDisplayName($row['name']), true);
			$retVal .= tag('severity', $row['severity']);
			$retVal .= tag('msg', $row['msg'], true);
			$retVal .= tag('warranty', $row['warrantyStatus'], true);
			$retVal .= tag('time', $row['time'], true);
			$retVal .= tag('hostid', $row['hostid']);
			$retVal .= tag('ip', $row['ip']);
			$retVal .= tag('alarmDefId', $row['alarmDefId']);
			$retVal .= tag('serialno', $row['serialno'], true);
			$retVal .= tag('deviceName', $row['deviceName'], true);
			$retVal .= tag('mac', $row['mac'], true);
			$retVal .= tag('network', $row['network'], true);


			$retVal .= "</alarm>\n";
			$i++;
		}

		$retVal = tag('alarms', $retVal);
		//$retVal .= tag('totalCount', $totalRecords); 
		$retVal .= tag('hasActive', $hasActive);
		return $retVal;

	}

	/**
	 *
	 */
	function getAlarmCounts()
	{
		$query = <<<EOQ
SELECT
	SUM(IF(severity = 'critical', 1, 0)) AS critical,
	SUM(IF(severity = 'warning', 1, 0)) AS warning,
	SUM(IF(severity = 'info', 1, 0)) AS info,
	SUM(IF(ActiveAlarm.id IS NULL, 0, 1)) AS active
FROM
	Alarms
	INNER JOIN Hosts ON
		Alarms.hostId = Hosts.Id
	INNER JOIN AlarmProfile2AlarmDef ON
		Alarms.alarmId = AlarmProfile2AlarmDef.alarmId
		AND Hosts.alarmProfileId = AlarmProfile2AlarmDef.alarmProfileId
	LEFT JOIN ActiveAlarm ON
		Alarms.id = ActiveAlarm.alarm_id
WHERE
	NOT cleared
EOQ;

		$connection = @getConnection();
		
		$result = @mysql_query($query, $connection);
		
		$summary = mysql_fetch_object($result);
		
		$xml = "<alarmCount>"
			. "<critical>{$summary->critical}</critical>"
			. "<warning>{$summary->warning}</warning>"
			. "<info>{$summary->info}</info>"
			. "<active>{$summary->active}</active>"
			. "</alarmCount>";
		
		
		mysql_free_result($result);
		
		return $xml;
	}

	function removeAlarms($alarmIdStr)
	{
		if (!preg_match("/^\\d+(?:\\|\\d+)*$/", $alarmIdStr))
		{
			error("Alarm list parameter format error");
		}
		
		$alarmIdStr = str_replace("|", ",", $alarmIdStr);
		
		$query = "DELETE FROM Alarms WHERE id IN ($alarmIdStr)";
		
		@sqlUpdate($query);
		$error = mysql_error();
		if ($error != "")
		{
			error("Unable to delete alarms: $error");
		}

		return tag('result', 'success');
	}

	/*********************************************************
	 * Clear the selected alarms.
	 * $alarmIdStr = id|id|id
	 */
	function clearAlarms($alarmIdStr)
	{
		$alarmIds = explode('|', $alarmIdStr);

		foreach ($alarmIds as $alarmId)
		{
			$alarmId = (int) $alarmId;

			if ($alarmId <= 0) continue;

			$query = "UPDATE Alarms set Cleared = 1 "
					. " WHERE id = $alarmId ";

			@sqlUpdate($query);
			$error = mysql_error();
			if ($error != "")
				error("Unable to update alarm! $error");

		}

		return tag('result', 'success');
	}

	/*********************************************************
	 * Ignore the selected alarms until the given end time
	 * $alarmIdStr = hostId.alarmId|hostId.alarmId|hostId.alarmId
	 * The client takes care of informing user that date is
	 * in the past if this is the case.
	 */
	function ignoreAlarms($alarmIdStr, $date)
	{
		$pairs = explode('|', $alarmIdStr);


		foreach ($pairs as $pair)
		{

			if ($pair == "") continue;

			if (!strpos($pair, ".")) continue;

			$pairValues = explode(".", $pair);

			if (count($pairValues) != 2) continue;

			$hostId = (int) $pairValues[0];
			$alarmDefId = (int) $pairValues[1];

			$query = "UPDATE Host2Alarm SET ignoreEndTime = '$date' "
					. " WHERE alarmId = $alarmDefId AND hostId = $hostId ";


			@sqlUpdate($query);

			$error = mysql_error();
			if ($error != "")
				error("Unable to update alarm! $error");

		}

		return tag('result', 'success');
	}

	/*********************************************************
	 * Stop ignoring the given alarms
	 *
	 */
	function stopIgnoring($alarmIdStr)
	{
		$pairs = explode('|', $alarmIdStr);

		foreach ($pairs as $pair)
		{

			if ($pair == "") continue;

			if (!strpos($pair, ".")) continue;

			$pairValues = explode(".", $pair);

			if (count($pairValues) != 2) continue;

			$hostId = (int) $pairValues[0];
			$alarmDefId = (int) $pairValues[1];

			$query = "UPDATE Host2Alarm SET ignoreEndTime = NULL "
					. " WHERE alarmId = $alarmDefId AND hostId = $hostId ";


			@sqlUpdate($query);

			$error = mysql_error();
			if ($error != "")
				error("Unable to update alarm! $error");

		}

		return tag('result', 'success');
	}

	/*********************************************************
	 * Get the list of alarms that are currently being ignored.
	 *
	 */
	function getIgnoredAlarms()
	{
		global $currentUserId;

		$pageNumber = (int) req('pageNumber');
		$pageSize = (int) req('pageSize');
		$page = ($pageNumber - 1) * $pageSize;

		$isRestricted = isUserNetworkUser($currentUserId);
		$networkids = array();

		if ($isRestricted)
			$networkids = getAllowableNetworkIds($currentUserId);

		$retVal = "";

		$recordCountQuery = " SELECT Count(AlarmDefinition.id) as Total ";

		$query = " SELECT "
				. " AlarmProfile2AlarmDef.severity,  "
				. "  AlarmDefinition.id as alarmDefId,  "
				. "  AlarmDefinition.name,  "
				. "  AlarmDefinition.threshold,  "
				. "  AlarmDefinition.oid,  "
				. "  Host2Alarm.ignoreEndTime, "
				. "  Hosts.id as hostid,  "
				. "  Hosts.serialno,  "
				. "  INET_NTOA(Hosts.ipaddr) as ip,  "
				. "  Hosts.name as deviceName,  "
				. "  Hosts.mac,  "
				. "  Networks.name as network  ";

		$subquery = "  FROM Host2Alarm, AlarmDefinition, AlarmProfile2AlarmDef, Hosts,  "
				. "  Networks  "
				. "  WHERE  "
				. "  Host2Alarm.alarmId = AlarmDefinition.id "
				. "  AND Host2Alarm.hostid = Hosts.id  "
				. "  AND AlarmProfile2AlarmDef.alarmid = AlarmDefinition.id  "
				. "  AND Hosts.alarmProfileId = AlarmProfile2AlarmDef.alarmProfileId  "
				. "  AND Networks.id = Hosts.networkId  "
				//. "  AND AlarmProfile2AlarmDef.deleted = 0  "
				. "  AND Host2Alarm.ignoreEndTime > NOW()";


		if ($isRestricted)
		{
			$subquery .= " AND (0 ";

			$i = 0;
			foreach ($networkids as $networkid)
			{
				$subquery .= " OR ";

				$subquery .= " Networks.id = $networkid ";

				$i++;
			}
			$subquery .= ")";

		}

		$recordCountQuery .= $subquery;
		$query .= $subquery;

		$query .= " LIMIT $page, $pageSize ";

		$totalRecords = @sqlNumber($recordCountQuery, "Total");

		$conn = @getConnection();

		$result = mysql_query($query, $conn);

		if (!$result)
			error("Unable to retrieve ignored alarms from the database! " . mysql_error());

		$numrows = mysql_numrows($result);

		$i = 0;
		$list = '';

		while ($i < $numrows)
		{
			$row = mysql_fetch_assoc($result);

			$list .= getStdAlarmDefDisplayName($row['name']);

			$retVal .= "<ignoredAlarm>\n";

			//$retVal .= tag('name', $row['name'], true);
			$retVal .= tag('name', getStdAlarmDefDisplayName($row['name']), true);
			$retVal .= tag('severity', $row['severity']);
			$retVal .= tag('threshold', $row['threshold']);
			$retVal .= tag('oid', $row['oid'], true);
			$retVal .= tag('ignoreEndTime', $row['ignoreEndTime'], true);
			$retVal .= tag('hostid', $row['hostid']);
			$retVal .= tag('alarmDefId', $row['alarmDefId']);
			$retVal .= tag('ip', $row['ip']);
			$retVal .= tag('serialno', $row['serialno'], true);
			$retVal .= tag('deviceName', $row['deviceName'], true);
			$retVal .= tag('mac', $row['mac'], true);
			$retVal .= tag('network', $row['network'], true);

			$retVal .= "</ignoredAlarm>\n";
			$i++;
		}

		$retVal = tag('ignoredAlarms', $retVal);
		$retVal .= tag('totalCount', $totalRecords);

		return $retVal;
	}

	function  stopIgnoringAlarm($alarmDefId, $hostId)
	{
		$alarmDefId = (int) $alarmDefId;
		$hostId = (int) $hostId;

		$query = "UPDATE Host2Alarm SET ignoreEndTime = NULL "
				. " WHERE alarmId = $alarmDefId AND hostId = $hostId ";

		@sqlUpdate($query);

		$error = mysql_error();

		if ($error != "")
			error("Unable to alter alarms! $error");

		return tag('result', 'success');
	}


	/********************************************************************
	 * Clear out all alarms in the system!
	 *
	 */
	function clearAllAlarms($hostId, $onlyNonActive = false)
	{

		$hostId = (int) $hostId;

		global $currentUserId;

		$networkQuery = "";

		if (!isUserAdminUser($currentUserId))
		{

			$networkids = getAllowableNetworkIds($currentUserId);

			$networkQuery .= " AND  (0 ";

			foreach ($networkids as $networkid)
			{
				$networkid = (int) $networkid;

				$networkQuery .= " OR  Hosts.networkId = $networkid ";

			}

			$networkQuery .= ") ";
		}

		$query = "UPDATE Alarms, Hosts "
				. " SET Alarms.cleared = 1 "
				. " WHERE Alarms.cleared = 0 "
				. " AND Alarms.hostId = Hosts.id "
				. $networkQuery
				. ($hostId > 0 ? " AND Alarms.hostid = $hostId " : "")
				. ($onlyNonActive ? " AND (Alarms.id not in  "
						. "       (SELECT alarm_id FROM ActiveAlarm WHERE ActiveAlarm.alarm_id = Alarms.id)) "
						: "")
				. "   ";


		@sqlUpdate($query);

		sqlError("Unable to clear out alarms! ");

		return tag('result', 'success');

	}

	/********************************************************************
	 * Clear out all alarms in the system!
	 *
	 */
	function clearNonActive($hostId)
	{
		return clearAllAlarms($hostId, true);

	}

	/**************************************************************
	 * Translate the client's sort by field to actual DB values
	 *
	 **************************************************************/
	function getSortByField_alarms($name)
	{

		$name = strtolower($name);

		switch ($name)
		{
			case "time" :
				return "Alarms.time";
			case "name" :
				return "AlarmDefinition.name";
			case "@status" :
				return "Alarms.status";
			case "severity" :
				return "AlarmProfile2AlarmDef.severity";
			case "devicename" :
				return "Hosts.name";
			case "network" :
				return "Networks.name";
			case "ip" :
				return "INET_NTOA(Hosts.ipaddr)";
			case "msg" :
				return "Alarms.msg";
		}

		return "";

	}
}
?>
