<?php

include('lib/sql.php');
include('lib/user.php');
include('lib/permissions.php');
include('lib/alarms.php');
include('lib/stats.php');
include('lib/system-time.php');

if (!function_exists('dashboard_wrapper'))
{

	function dashboard_wrapper()
	{
	}


	/**************************************************************
	 * Handle dashboard action
	 *
	 */
	function handleDashboardAction($action)
	{
		if ($action == null)
			error('Unauthorized');

		switch ($action)
		{
			case 'get':
				echo getDashboardInfo(req('maxAlerts'), req('alertTypes'),
					req('showClosed'), req('showNotify'));
				break;

		}
	}

	/**************************************************************
	 * Get info shown on dashboard
	 * $maxAlerts = max number of alerts to return
	 * $alertTypes = types to include (ex: crtical|warning)
	 */
	function getDashboardInfo($maxAlerts, $s_alertTypes, $showClosed, $showNotify)
	{
		$maxAlerts = (int) $maxAlerts;
		global $currentUserId;

		$retVal = "<results><dashboard>";

		$alertTypes = explode('|', $s_alertTypes);

		/* 1. Current tasks.
		 * 2. Number of devices up and down
		 * 3. Number of registered and unregistered devices
 		 * 4. Latest alerts
		 * 5. System status
		 * 6. Custom monitoring stuff (TBD)
		 * 7. Total alert count (unfiltered!)
		*/

		$isRestricted = isUserNetworkUser($currentUserId);

		$restrictedQuery = " AND Networks.id = Hosts.networkId "
				. " AND Networks.id = User2Network.networkId "
				. " AND User2Network.userid = $currentUserId ";

		$query = " SELECT "
				// Down devices
				. " (SELECT Count(Hosts.id) FROM Hosts "
				. ($isRestricted ? " , User2Network, Networks " : " ")
				. " WHERE Hosts.IsDown = 0 "
				. ($isRestricted ? $restrictedQuery : " ")
				. " ) as up, "
				// Up devices
				. " (SELECT Count(Hosts.id) FROM Hosts "
				. ($isRestricted ? " , User2Network, Networks " : " ")
				. " WHERE Hosts.IsDown > 0 "
				. ($isRestricted ? $restrictedQuery : " ")
				. " ) as down, "
				// Unregistered devices
				. " (SELECT Count(Hosts.id) FROM Hosts "
				. ($isRestricted ? " , User2Network, Networks " : " ")
				. " WHERE Hosts.Registered = 0 "
				. ($isRestricted ? $restrictedQuery : " ")
				. " ) as unregistered, "
				// Registered
				. " (SELECT Count(Hosts.id) FROM Hosts "
				. ($isRestricted ? " , User2Network, Networks " : " ")
				. " WHERE Hosts.Registered > 0 "
				. ($isRestricted ? $restrictedQuery : " ")
				. " ) as registered ";


		$deviceStatus = sqlRow($query);

		$retVal .= tag('up', $deviceStatus['up']);
		$retVal .= tag('down', $deviceStatus['down']);
		$retVal .= tag('registered', $deviceStatus['registered']);
		$retVal .= tag('unregistered', $deviceStatus['unregistered']);

		$retVal .= "<system>\n";
		
		$isWindows = isWindows();
		$release = php_uname('r');

		if ($isWindows && (strstr($release, '5.1') || strstr($release, '5.2')))
		{
			$uptimeCommand = dirname(__FILE__) . '/../../../../wnms/uptime.exe';
			$line = `$uptimeCommand`;
		}
		else
		{
			$line = shell_exec('uptime');
		}

		if ($isWindows)
		{

			$auptime = explode('for: ', $line);
			$auptime = explode(',', $auptime[1]);

			$days = str_replace('day(s)', 'days', $auptime[0]);

			$hours = str_replace('hour(s)', 'hrs', $auptime[1]);

			$mins = str_replace('minute(s)', 'mins', $auptime[2]);
			$secs = str_replace('second(s)', 'secs', $auptime[3]);

			$uptime = "$days, $hours, $mins, $secs";

			$retVal .= tag('uptime', $uptime);

		}
		else
		{

			$auptime = explode(' up ', $line);
			$auptime = explode(',', $auptime[1]);
			$uptime = $auptime[0];

			if (strpos($auptime[1], 'users') < 0)
				$uptime .= ', ' . $auptime[1];

			$retVal .= tag('uptime', $uptime);

		}
		
		$systemStatus = @sqlString("SELECT `text` AS status FROM TranslatedPhrases WHERE id=900", "status");

		// XXX TODO: what would go here otherwise???
		$retVal .= tag('status', $systemStatus);

		$retVal .= "</system>";


		/*  Let's get task stats */

		$networkFilter = "";

		if ($isRestricted)
		{
			$networkFilter = " AND (0 ";
			$allowedNetworkIds = getAllowableNetworkIds($currentUserId);

			foreach ($allowedNetworkIds as $networkId)
			{
				$networkId = (int) $networkId;

				$networkFilter .= " OR Hosts.networkId = $networkId ";

			}
			$networkFilter .= ")";
		}

		$restrictedNetworkQuery = " AND (SELECT Count(Host2Task.idhost) FROM Host2Task  "
				. " WHERE Host2Task.idtask = t.id) "
				. " =  "
				. " (SELECT Count(Host2Task.idhost) FROM Host2Task, Hosts, User2Network  "
				. " WHERE Host2Task.idTask = t.id  "
				. " AND User2Network.networkId = Hosts.networkId "
				. " AND User2Network.userId = $currentUserId "
				. " AND Hosts.id = Host2Task.idhost   $networkFilter )";
		$query = "SELECT  "
				// Completed tasks
				. " ("
				. " SELECT Count(t.id) "
				. " FROM Task t "
				. " WHERE t.cleared = 0 "
				. " AND (SELECT Count(idhost) FROM Host2Task "
				. "      WHERE Host2Task.idtask = t.id "
				. "      AND Host2Task.dateCompleted is not NULL) "
				. " 	 =    "
				. "     (SELECT Count(idhost) from Host2Task "
				. "      WHERE Host2Task.idtask = t.id) "

				. ($isRestricted ? $restrictedNetworkQuery : "")

				. ")As completed, "


				// Now scheduled tasks:
				. "("
				. " SELECT Count(t.id)  "
				. " FROM Task t "
				. " WHERE t.cleared = 0 "
				. " AND (SELECT Count(idhost) FROM Host2Task "
				. "      WHERE Host2Task.idtask = t.id "
				. "      AND Host2Task.dateCompleted is NULL) > 0 "

				. ($isRestricted ? $restrictedNetworkQuery : "")


				. ") As scheduled,"
				// Now failed tasks:
				. "("
				. " SELECT Count(t.id)  "
				. " FROM Task t "
				. " WHERE t.cleared = 0 "
				. " AND (SELECT Count(idhost) FROM Host2Task "
				. "       WHERE Host2Task.idtask = t.id  "
				. "       AND Host2Task.status like 'fail%') > 0 "

				. ($isRestricted ? $restrictedNetworkQuery : "")

				. ") As failed ";

		$retVal .= "<tasks>";

		$taskStats = sqlRow($query);

		$retVal .= tag('completed', $taskStats['completed']);
		$retVal .= tag('failed', $taskStats['failed']);
		$retVal .= tag('scheduled', $taskStats['scheduled']);

		$retVal .= "</tasks>";

		/* Get the stats graphs to show on the dashboard */
		$retVal .= getDashboardStatsData($currentUserId);

		$retVal .= getDashboardAlarms($maxAlerts, $alertTypes, $showClosed, $showNotify);

		// only active alarms
		$badAlarms = getDashboardAlarms(1, array("critical", "warning"), 0, 0);

		$dom = new DOMDocument();
		$dom->loadXML('<result>' . $badAlarms . '</result>');

		// Let's see if we have any active alerts (NOT notify), so we
		// know whether or not to show the alert icon on the Alerts tab
		// in the gui.

		$hasActive = $dom->getElementsByTagName('hasActive')->item(0)->nodeValue;
		$hasActive = (int) $hasActive;

		$retVal .= tag('hasAlerts', $hasActive);

		// Add extra functionality, not breaking existing alarms - count all unfiltered alarms
		$retVal .= getAlarmCounts();

		$retVal .= "</dashboard>";

		$retVal .= "</results>";
		return $retVal;

	}


}

?>
