<?php

if (!function_exists('ipaddr_utils_wrapper')) {

  function ipaddr_utils_wrapper()
  {
  }


  function _isLoopbackAddress($ip)
  {
    return ($ip & 0xff000000) === (int) 0x7f000000;
  }
  
  function _isLANAddress($ip)
  {
    if ($ip === FALSE) {
      return FALSE;
    }
   
    $isLAN = (
      ($ip & 0xff000000) === (int) 0x0a000000 || // 10.x.x.x
      ($ip & 0xfff00000) === (int) 0xac100000 || // 172.16.x.x - 172.31.x.x
      ($ip & 0xffff0000) === (int) 0xc0a80000    // 192.168.x.x
    );
    
    return $isLAN;
  }
  
  /*
   * Returns TRUE if $ip is a loopback IP address, FALSE otherwise.
   */
  function isLoopbackAddress($addrSpec) {
    return _isLoopbackAddress(ip2long($addrSpec));
  }
  
  /*
   * Returns TRUE if $ip is a LAN IP address, FALSE otherwise.
   */
  function isLANAddress($addrSpec) {
    return _isLANAddress(ip2long($addrSpec));
  }
  
  /*
   * Returns TRUE if $ip is a local (loopback or LAN) IP address, FALSE
   * otherwise.
   */
  function isLocalAddress($addrSpec) {
    $ip = ip2long($addrSpec);
    return _isLoopbackAddress($ip) || _isLANAddress($ip);
  }
  
  /*
   * Looks up the server's WAN IP address by querying a remote web
   * service. Has security implications if we're using a 3rd party
   * service. However, lib/tasks.php::generateURL() calls
   * lib/verify-utils.php::verifyServerURL() that makes sure the
   * URL points back to the server.
   *
   * Returns the IP address as a string or FALSE on error.
   */
  function lookupWANAddress()
  {
    $serviceURLs = array (
      'http://www.wilibox.com/web-services/ip-address',
      'http://www.whatismyip.com/automation/n09230945.asp'
    );
    
    foreach ($serviceURLs as $url) {
      $addrSpec = @file_get_contents($url);
      
      if ($addrSpec !== FALSE && ip2long($addrSpec) !== FALSE) {
        return $addrSpec;
      }
 
    }
    
    trigger_error("unable to determine server's WAN IP address: none of the web services worked");
    return FALSE;
  }
  
  
  function lookupLANAddress()
  {
    $hostSpec = gethostname();
    
    if ($hostSpec === FALSE) {
      trigger_error("unable to determine server's hostname");
      return FALSE;
    }
    
    $hostAddr = gethostbyname($hostSpec);
    
    if ($hostAddr === FALSE) {
      trigger_error("unable to lookup IP address for hostname $hostSpec");
      return FALSE;
    }
    
    /* This will NOT necessarily be a LAN address! */
    return $hostAddr;
  }
  
  define ('LOOKUP_LAN_ADDRESS', 0x00000001);
  define ('LOOKUP_WAN_ADDRESS', 0x00000002);

  function detectServerAddress($flags)
  {
    $hostSpec = $_SERVER['SERVER_NAME'];
    $addrSpec = $_SERVER['SERVER_ADDR'];
    
    /* Do they need the WAN address? */
    
    if ((int) $flags & (int) LOOKUP_WAN_ADDRESS) {
      if (!isLocalAddress($addrSpec)) {
        return $hostSpec;
      }
      
      $wanAddr = lookupWANAddress();
      
      if ($wanAddr !== FALSE || ((int) $flags & (int) LOOKUP_LAN_ADDRESS) == 0) {
        return $wanAddr;
      }
    }
    
    /* Are they OK with a LAN address? */
    
    if ((int) $flags & (int) LOOKUP_LAN_ADDRESS) {
      if (!isLoopbackAddress($addrSpec)) {
        return $addrSpec;
      }
      $lanAddr = lookupLANAddress();
      return $lanAddr;
    }
    
    return $addrSpec;
  }
  
  function detectServerAddressAndPort($flags = 0)
  {
    $serverAddr = detectServerAddress($flags);
    
    if ($serverAddr === FALSE) {
      return FALSE;
    }
    
    $serverPort = $_SERVER['SERVER_PORT'];
    
    if ($serverPort !== '80') {
      $serverAddr .= ':';
      $serverAddr .= $serverPort;
    }
    
    return $serverAddr;
  }

}

?>
