<?php

include('lib/sql.php');
include('lib/user.php');
include('lib/permissions.php');

if(!function_exists('maps_wrapper'))
{

	function maps_wrapper() {}

	
	function handleMapsAction($action)
	{
		if($action == null)
			error('Unauthorized');

		switch($action)
		{
			case 'add-hosts':
			echo addHostsToMap(req('hosts'));
			break;

			case 'remove-hosts':
			echo removeHostsFromMap(req('hosts'));
			break;

			case 'get':
			echo getMapData(req('networks'));
			
			break;
			case 'host-location':
			echo updateHostLocation(req('hostId'), req('lat'), req('lon'), req('clearLocation'));

			break;

			case 'network-location':
			echo updateNetworkLocation(req('networkId'), req('lat'), req('lon'), req('clearLocation'));

			break; 
		}

	}

	/*****************************************************************
	* Update the location of a host in the DB
	*
	*/
	function updateHostLocation($hostId, $lat, $lon, $clearLocation)
	{
		$lat = (double) $lat;
		$lon = (double) $lon;
		$hostId = (int) $hostId;

		if(!$clearLocation)
		$query = "UPDATE Hosts Set latitude_local  = $lat, longitude_local  = $lon "
			. " WHERE id = $hostId ";

		else
		$query = "UPDATE Hosts Set latitude_local   = NULL, longitude_local  = NULL "
			. " WHERE id = $hostId ";

		@sqlUpdate($query);

		sqlError("Unable to update host location! ");

		return tag('result', 'success');
	}


	/*****************************************************************
	* Update the location of a network in the DB
	*
	*/
	function updateNetworkLocation($networkId, $lat, $lon, $clearLocation)
	{
		$lat = (double) $lat;
		$lon = (double) $lon;
		$networkId = (int) $networkId;

		if(!$clearLocation)
		$query = "UPDATE Networks Set centerLat  = $lat, centerLon = $lon "
			. " WHERE id = $networkId ";

		else
		$query = "UPDATE Networks Set centerLat  = NULL, centerLon = NULL "
			. " WHERE id = $networkId ";

		@sqlUpdate($query);

		sqlError("Unable to update network location! ");

		return tag('result', 'success');
	}

	/********************************************************
	*  Get all devices, links and network info for the 
	*  give networks
   *  $networkIds = "id,id,id," etc...
	* 
	*/
	function getMapData($networkIdStr)
	{
		global $currentUserId;

		$networkIds = explode(',', $networkIdStr );
		
		// Only include networks that this user is allowed to access		
		$isNetworkUser = isUserNetworkUser($currentUserId);

		if($isNetworkUser)
		{
			$allowedNetworks = getAllowableNetworkIds($currentUserId);

			/* If we've not specified any network ids (meaning search them all,
			* then make sure we only search the ones that belong to the given user
			*/
			if($networkIdStr == "" || !is_array($networkIds) || count($networkIds ==0))
				$networkIds = $allowedNetworks;
		}


		$networkQuery = getNetworkFilterQuery("Networks.id", $networkIds, $isNetworkUser, true, $allowedNetworks);
		

		/* Get networks */
		
		$retVal = "<map>";

		$query = "SELECT Networks.id, Networks.name, Networks.centerLat as lat, "
			. " Networks.centerLon as lon, Networks.type, "
			// Uncleared count (both active +

			.  " CASE WHEN Exists   (SELECT Hosts.id "
			.  "      FROM Hosts,AlarmProfile2AlarmDef, Alarms  "
			.  "     WHERE  "
			.  "     Hosts.networkid = Networks.id "
			.  "   	AND Alarms.cleared = 0 "
			.  "   	AND Alarms.hostId = Hosts.id "
			.  "   	AND Alarms.alarmId = AlarmProfile2AlarmDef.alarmId "
			.  "   	AND AlarmProfile2AlarmDef.alarmProfileId = Hosts.alarmProfileId  "
			.  "   	AND (AlarmProfile2AlarmDef.severity = 'warning' OR "
			.  "   	 	  AlarmProfile2AlarmDef.severity = 'critical') "
			. 	"     ) THEN 1 ELSE 0 END AS  hasUncleared, "


			// has warning
			.  " CASE WHEN Exists   (SELECT Hosts.id "
			.  "      FROM Hosts,AlarmProfile2AlarmDef, Alarms  "
			. 	" 		Left Join ActiveAlarm on Alarms.id = ActiveAlarm.alarm_id "
			.  "     WHERE  "
			.  "     Hosts.networkid = Networks.id "
			. 	"		AND (ActiveAlarm.alarm_id is not null) "
			.  "   	AND Alarms.cleared = 0 "
			.  "   	AND Alarms.hostId = Hosts.id "
			.  "   	AND Alarms.alarmId = AlarmProfile2AlarmDef.alarmId "
			.  "   	AND AlarmProfile2AlarmDef.alarmProfileId = Hosts.alarmProfileId  "
			.  "   	AND (AlarmProfile2AlarmDef.severity = 'warning' ) "
			. 	"     ) THEN 1 ELSE 0 END AS  hasWarning, "

			// has critical 
		.  " CASE WHEN Exists   (SELECT Hosts.id "
			.  "      FROM Hosts,AlarmProfile2AlarmDef, Alarms  "
			. 	" 		Left Join ActiveAlarm on Alarms.id = ActiveAlarm.alarm_id "
			.  "     WHERE  "
			.  "     Hosts.networkid = Networks.id "
			. 	"		AND (ActiveAlarm.alarm_id is not null) "
			.  "   	AND Alarms.cleared = 0 "
			.  "   	AND Alarms.hostId = Hosts.id "
			.  "   	AND Alarms.alarmId = AlarmProfile2AlarmDef.alarmId "
			.  "   	AND AlarmProfile2AlarmDef.alarmProfileId = Hosts.alarmProfileId  "
			.  "   	AND (AlarmProfile2AlarmDef.severity = 'critical') "
			. 	"     ) THEN 1 ELSE 0 END AS  hasCritical, "


			. " (SELECT max(Hosts.latitude_local ) FROM Hosts WHERE Hosts.networkId = Networks.id AND Hosts.latitude_local  != 0) as maxLat, "
			. " (SELECT min(Hosts.latitude_local ) FROM Hosts WHERE Hosts.networkId = Networks.id AND Hosts.latitude_local  != 0) as minLat, "
			. " (SELECT max(Hosts.longitude_local )  FROM Hosts WHERE Hosts.networkId = Networks.id AND Hosts.longitude_local  != 0) as maxLon, "
			. " (SELECT min(Hosts.longitude_local )  FROM Hosts WHERE Hosts.networkId = Networks.id AND Hosts.longitude_local  != 0) as minLon, "
		   . " (SELECT COUNT(id) FROM  Hosts "
			. "     WHERE Hosts.networkid = Networks.id ) as devices "
			. " FROM Networks "
			. " WHERE 1 ";
		
		$query .= $networkQuery;

		
		$conn = @getConnection();

		$result = mysql_query($query, $conn);

		if(!$result)
		{
			error("Unable to retrieve map data! ". mysql_error());
			return;
		}

		$numrows = mysql_numrows($result);
		$i = 0;

		$retVal .= "<networks>";

		while($i < $numrows)
		{	
			$row = mysql_fetch_assoc($result);
			
			$id = (int) $row['id'];

			$lat = (double) $row['lat'];
			$lon = (double) $row['lon'];

			$devices = (int)$row['devices'];
			$type = getReadableNetworkType($row['type']);
			$minLat = (double) $row['minLat'];
			$minLon = (double) $row['minLon'];
			$maxLat = (double) $row['maxLat'];
			$maxLon = (double) $row['MaxLon'];

			$critical = (int) $row['hasCritical'];
			$warning = (int) $row['hasWarning'];
			
			$status = "ok";

			if($critical)
				$status = "critical";
			else if($warning)
				$status = "warning";
	
			$hasUncleared = (int) $row['hasUncleared'];

			if($lat == 0 && $lon == 0)
				$valid = 0;
			else $valid = 1;

			$network = tag('name', $row['name']);
				
			$att = "id='$id' lat='$lat' devices='$devices' lon='$lon' type='$type' valid='$valid'";
			$att .= " minLat='$minLat' minLon='$minLon' maxLat='$maxLat' maxLon='$maxLon' ";
			$att .= " status='$status' hasUncleared='$hasUncleared' ";

			$retVal .= tag("n", $network, FALSE, $att);			

			$i++;

		}
		$retVal .= "</networks>";

		/* Now get hosts... 	*/


		$query = "SELECT Hosts.id, Hosts.deviceTypeId, Hosts.name, IFNULL(Hosts.latitude_local, Hosts.latitude) as lat, "
			. " IFNULL(Hosts.longitude_local, Hosts.longitude) as lon, Hosts.isdown, INET_NTOA(Hosts.ipaddr) as ip, Hosts.mac, "
			. " IF(networkId IS NULL AND firmware LIKE 'SWEAP v%', 1, 0) AS isSweap,"
		
			. " (SELECT Count(Alarms.id) as totalCritical "
			. "  FROM Alarms "
			. "  Left Join AlarmProfile2AlarmDef "
			. "  ON Alarms.alarmId = AlarmProfile2AlarmDef.alarmId "
			. "  Left Join ActiveAlarm ON ActiveAlarm.alarm_id = Alarms.id "
			. "  WHERE Alarms.cleared = 0 "
			. "  AND ActiveAlarm.alarm_id is not null "
			. "  AND AlarmProfile2AlarmDef.alarmProfileId = Hosts.alarmProfileId "
			. "  AND AlarmProfile2AlarmDef.severity = 'critical' "
			. "  AND Alarms.hostid = Hosts.id ) as totalCritical, "
		 
			. " (SELECT Count(Alarms.id) as totalWarning "
			. "  FROM Alarms "
			. "  Left Join AlarmProfile2AlarmDef "
			. "  ON Alarms.alarmId = AlarmProfile2AlarmDef.alarmId "
			. "  Left Join ActiveAlarm ON ActiveAlarm.alarm_id = Alarms.id "
			. "  WHERE Alarms.cleared = 0 "
			. "  AND ActiveAlarm.alarm_id is not null "
			. "  AND AlarmProfile2AlarmDef.alarmProfileId = Hosts.alarmProfileId  "
			. "  AND AlarmProfile2AlarmDef.severity = 'warning' "
			. "  AND Alarms.hostid = Hosts.id ) as totalWarning, "
			
 			. "  CASE WHEN Exists (SELECT Alarms.id  "
			. "  FROM Alarms "
			. "  Left Join AlarmProfile2AlarmDef "
			. "  ON Alarms.alarmId = AlarmProfile2AlarmDef.alarmId "
			. "  WHERE Alarms.cleared = 0 "
			. "  AND AlarmProfile2AlarmDef.alarmProfileId = Hosts.alarmProfileId  "
			. "  AND (AlarmProfile2AlarmDef.severity = 'warning' OR AlarmProfile2AlarmDef.severity = 'critical') "
			. "  AND Alarms.hostid = Hosts.id Limit 1) THEN 1 ELSE 0 END as hasUncleared , "


			. " CASE WHEN Networks.name is null then 'No Network' ELSE Networks.name END as network "
			. " FROM Hosts Left Join Networks "
			. " ON Hosts.networkId = Networks.id "
			. " WHERE Hosts.map = 1 ";
		
		$networkQuery = getNetworkFilterQuery("Hosts.networkid", $networkIds, $isNetworkUser, true, $allowedNetworks);

		$query .= $networkQuery;

		
		$conn = @getConnection();

		$result = mysql_query($query, $conn);

		if(!$result)
		{
			error("Unable to retrieve hosts for map data! ". mysql_error());
			return;
		}

		$numrows = mysql_numrows($result);
		$i = 0;

		$retVal .= "<hosts>";

		while($i < $numrows)
		{	
			$row = mysql_fetch_assoc($result);
			
			$id = (int) $row['id'];

			$lat = (double) $row['lat'];
			$lon = (double) $row['lon'];

			$warning = (int) $row['totalWarning'];
			$critical = (int) $row['totalCritical'];
			$down = (int) $row['isdown'];
			$did = (int) $row['deviceTypeId'];
			$hasUncleared = (int) $row['hasUncleared'];
			$isSweap = (int) $row['isSweap'];

			$status = "ok";

			if ($down > 0 || $critical > 0)
			{
				$status = "critical";
			}
			else if($warning > 0)
			{
				$status = "warning";
			}

			if (!is_numeric($row['lat']) || !is_numeric($row['lon']))
			{
				$valid = 0;
			}
			else
			{
				$valid = 1;
			}

			/* Eh... dont show a link if it connects to a host that has no location */
			/* This can happen in cases where the link was added to the topo map */
			/* to a device whose location is not set */
			if($valid)
			{

				$host = tag('name', $row['name']);
				$host .= tag('network', $row['network']);
				$ip =$row['ip'];
				$mac = $row['mac'];
				
				$att = "id='$id' deviceTypeId='$did' status='$status' lat='$lat' lon='$lon' valid='$valid' "
					. " ip='$ip' mac='$mac' hasUncleared='$hasUncleared' isSweap='$isSweap'";

				$retVal .= tag("h", $host, FALSE, $att);		
			}	
			$i++;

		}
		$retVal .= "</hosts>";

 
		/* Now get links... */

		$query = "SELECT Links.id, Hosts1.id as fromId, "
				. " Hosts1.latitude_local  as lat1, "
				. " Hosts1.longitude_local  as lon1, "
				. " Hosts2.id as toId, "
				. " Hosts2.latitude_local  as lat2, "
				. " Hosts2.longitude_local  as lon2 "
				. " FROM  Hosts as Hosts1 "
				. " Left Join Links "
				. " on Links.fromHostId = Hosts1.id "
				. " Left Join Hosts as Hosts2 "
				. " ON Hosts2.id = Links.toHostId  "
				. " WHERE Hosts1.map = 1 "
				. " AND (Hosts2.map is null OR Hosts2.map = 1) ";

	
		$networkQuery = getNetworkFilterQuery("Hosts1.networkid", $networkIds, $isNetworkUser, true, $allowedNetworks);
		$networkQuery .= getNetworkFilterQuery("Hosts2.networkid", $networkIds, $isNetworkUser, true, $allowedNetworks);

		if($networkQuery != "")
		{
			$query .= $networkQuery;
		}


		$conn = @getConnection();

		$result = mysql_query($query, $conn);

		if(!$result)
		{
			error("Unable to retrieve topo map links! ". mysql_error());

			return;
		}


		$numrows = mysql_numrows($result);

		$i = 0;
 
		$retVal .= "<links>";

		while($i < $numrows)
		{	

			$row = mysql_fetch_assoc($result);
			
			$id = (int) $row['id'];

			$lat1 = (double) $row['lat1'];
			$lon1 = (double) $row['lon1'];

			$lat2 = (double) $row['lat2'];
			$lon2 = (double) $row['lon2'];

			$fromId = (int) $row['fromId'];
			$toId = (int) $row['toId'];

			$valid = 1;

			if(($lat1 ==0  && $lon1 == 0) || ($lat2 == 0 && $lon2 ==0))
				$valid = 0;

			if($valid)
			{
				$att = "id='$id' lat1='$lat1' lon1='$lon1' lat2='$lat2' lon2='$lon2' "
						. " fromId='$fromId' toId='$toId' ";

				$retVal .= "<l $att />";			
			}
			$i++;
		}

		$retVal .= "</links>";

		$retVal .= "</map>";
		return $retVal;
	}

	function getReadableNetworkType($type)
	{

		switch($type)
		{		
			case 'ptp': return "Point-to-Point";
			case 'ptmp': return "Point to Multi Point";
			case 'mesh': return "Mesh";
			default: return "Other/Unknown";
		}
	}

	function getNetworkFilterQuery($prefix, $networkIds, $isNetworkUser, $includeNoNetwork=true,
											 $allowedNetworks=null)
	{	

		$networkQuery = "";

		if(is_array($networkIds) && count($networkIds) > 0)
		{	
			$networkQuery = " AND ( ";
			$networkSubQuery = "";

			foreach($networkIds as $networkId)
			{				
				$networkId = (int)$networkId;

				if($networkId == 0 && $includeNoNetwork)
				{
					if($networkSubQuery != "")
							$networkSubQuery .= " OR ";

					$networkSubQuery .= " $prefix = 0 OR $prefix is null ";

				}

				if($networkId <=0)
					continue;

				if(!$isNetworkUser || ($isNetworkUser && in_array($networkId, $allowedNetworks)))
				{
					if($networkSubQuery != "")
							$networkSubQuery .= " OR ";

					$networkSubQuery .= " $prefix = $networkId ";
				}
			}		

			$networkQuery .= $networkSubQuery . ")";
		}
		return $networkQuery;
	}

	/********************************************************
	*  Synch toplogy link changes with the database
	*  $changes = type,from link id, to link id | ....
	*  type = 0 or 1, 0 = remove, 1 = adds
	*/
	function syncChanges($changesStr)
	{
	
		$changes = explode('|', $changesStr);

		if(!is_array($changes))
			return tag('result', 'success');

		foreach($changes as $change)
		{

			$values = explode(',', $change);
			
			if(!is_array($values) || count($values) != 3)
				continue;
		
			$type = $values[0];
			$fromId = $values[1];
			$toId = $values[2];

			if($type == 0)
			{
				removeLink($fromId, $toId);
			}
			else if($type == 1)
			{
				addLink($fromId, $toId);
			}
			
	
		}

		return tag('result', 'success');
	}


	/********************************************************
	*  Remove hosts from the google map
	*/
	function removeHostsFromMap($hostString)
	{
		$hosts = explode(',', $hostString);

		if(!is_array($hosts))
			return tag('result', 'success');

		foreach($hosts as $hostId)
		{
			$hostId = (int) $hostId;

			// Remove all links to this host
			$query = "DELETE FROM Links WHERE fromHostId = $hostId "
					. " OR toHostId = $hostId ";

			@sqlUpdate($query);

			// Now remove from topology map 
			$query = "UPDATE Hosts set map = 0 " 
				. " WHERE id = $hostId ";

			@sqlUpdate($query);

			$error = mysql_error();

			if($error != "")
				error("Unable to remove host from the map! $error");
		}

		return tag('result', 'success');
	}

	/********************************************************
	*  Add hosts to the google map
	*/
	function addHostsToMap($hostString)
	{

		$hosts = explode(',', $hostString);

		if(!is_array($hosts))
			return tag('result', 'success');

		foreach($hosts as $hostId)
		{
			$hostId = (int) $hostId;

			$query = "UPDATE Hosts set map = 1 " 
				. " WHERE id = $hostId ";

			@sqlUpdate($query);

			$error = mysql_error();

			if($error != "")
				error("Unable to add hosts to the map! $error");
		}
		return tag('result', 'success');

	}

	/********************************************************
	*  Add a link between two hosts
	*/
	function addLink($fromId, $toId)
	{
		$fromId = (int) $fromId;
		$toId = (int) $toId;

		if($fromId == $toId || $fromId == 0|| $toId == 0)
			return;

		// first make sure this link doesn't already exist

		$query = "SELECT Links.id FROM Links "
				. " WHERE (toHostId = $toId AND fromHostId = $fromId) " 
				. " OR (toHostId = $fromId AND toHostId = $toId) ";

		$id = @sqlNumber($query, "id");

		if($id > 0)
		// already exist, so just return success
			return tag('result', 'success');

		$query  = "INSERT INTO Links "
				. " (toHostId, fromHostId) "
				. " VALUES "
				. " ($toId, $fromId)";

		@sqlInsert($query); 

		$error = mysql_error();

		if($error != "")
			error("Unable to add link! " . $error);

		return tag('result', 'success');
	}

	/********************************************************
	* This is run when use first logs in (b/c too much overhead
	* to do checks each query we do.
	*
	*/
	function setLocalLatLng()
	{
		$query = "UPDATE Hosts "
				. " SET latitude_local = latitude, "
				. " longitude_local = longitude "
				. " WHERE latitude_local is null AND longitude_local is null "
				. " AND latitude is not null and longitude is not null ";

		@sqlUpdate($query);

	}

	/********************************************************
	*  Remove a link between two hosts
	*/
	function removeLink($fromId, $toId)
	{
		$fromId = (int) $fromId;
		$toId = (int) $toId;

		$query  = "DELETE FROM Links "
				. " WHERE fromHostId = $fromId "
				. " AND toHostId = $toId ";

		@sqlCmd($query);

		$error = mysql_error();

		if($error != "")
			error("Unable to remove link! " . $error);
	}
}
?>
