<?php

include('lib/sql.php');
include('lib/permissions.php');
include('lib/user.php');

if(!function_exists('networks_wrapper'))
{

	function networks_wrapper() {}

	/* Lengths of DB fields in the Networks table */
	$NETWORK_LEN = array(
				'name' 			        => 25,
				'description' 	    => 100, 
				'locationLat' 	    => 25, 
				'locationLon' 	    => 25,	
				'ip'			  	      => 25,
				'type'			        => 25,
				'mask'			        => 25,
				'contactName'       => 100,
				'contactPhone'      => 50,
				'contactEmail'      => 100,
				'contactOtherInfo'  => 8192
				);


	
	function handleNetworkAction($action)
	{
		if($action == null)
			error('Unauthorized');

		switch($action)
		{
			case 'create':
				createNetwork(
					req('name'), req('description'),  req('ip'),
					req('mask'), req('locationLat'), req('locationLon'), 
					req('lostHeartbeatCount'),req('parentid'), 
					req('networkType'),
					req('contactName'),
					req('contactPhone'),
					req('contactEmail'),
					req('contactOtherInfo'),
					req('notifyContact')
			);
			break;
				
			case 'delete':
				deleteNetwork(req('networkid'));
	
			break;

			case 'changepass':				
				changePass(req('userid'), req('password'));
			break;

			case 'modify':
				modifyNetwork(
					req('networkid'), req('name'), req('description'),  req('ip'),
					req('mask'), req('locationLat'), req('locationLon'), 
					req('lostHeartbeatCount'),req('parentid'), 
					req('networkType'),
					req('contactName'),
					req('contactPhone'),
					req('contactEmail'),
					req('contactOtherInfo'),
					req('notifyContact')
				);
				break;
			
			case 'get':
				
				global $currentUserId;

				echo "<result><flat>" . getManageNetworksXML() . "</flat>"
					 ."<nested><networks>\n" . getSelectableNetworksXML($currentUserId) 
					. "</networks></nested></result>";
			break;

		}

	}

	/****************************************************
	* Create a network
	*
	**/
	function createNetwork($name, $description, $ip, $mask, $locationLat, 
								  $locationLon, $lostHeartbeatCount, $parentid, $networkType,
								  $contactName,
								  $contactPhone,
								  $contactEmail,
								  $contactOtherInfo,
								  $notifyContact)
	{

		global $currentUserId;
		global $NETWORK_LEN;

		$parentid = (int) $parentid;
		$lostHeartbeatCount = (int) $lostHeartbeatCount;
		if($lostHeartbeatCount == 0)
			$lostHeartbeatCount = 3;

		if(strlen($name) <1)
			error("Please specify a network name.");
	
		$query =   " INSERT INTO Networks "
					. " (name, description, parentid, changedBy, centerLat, "
					. " centerLon, lostHeartbeatThreshold, ip, netmask, type, "
					. " contactName, contactPhone, contactEmail, contactOtherInfo, notifyContact "
					. ") VALUES ("
					. " '" . cleanTrimParam($name, $NETWORK_LEN["name"])   	 	 . "', " 
					. " '" . cleanTrimParam($description, $NETWORK_LEN["description"]) . "', "
					. " $parentid, "
					. " $currentUserId, "
					. " '" . cleanTrimParam($locationLat, $NETWORK_LEN["locationLat"])  . "', " 
					. " '" . cleanTrimParam($locationLon, $NETWORK_LEN["locationLon"])  . "', " 
					. " $lostHeartbeatCount, " 
					. " '" . cleanTrimParam($ip, $NETWORK_LEN["ip"])  	. "', "
					. " '" . cleanTrimParam($mask, $NETWORK_LEN["mask"]) . "', "
					. " '" . cleanTrimParam($networkType, $NETWORK_LEN["type"]) . "', "
					. " '" . cleanTrimParam($contactName, $NETWORK_LEN["contactName"]) . "', "
					. " '" . cleanTrimParam($contactPhone, $NETWORK_LEN["contactPhone"]) . "', "
					. " '" . cleanTrimParam($contactEmail, $NETWORK_LEN["contactEmail"]) . "', "
					. " '" . cleanTrimParam($contactOtherInfo, $NETWORK_LEN["contactOtherInfo"]) . "', "
					. $notifyContact
					. " ) ";

		/* Get network id */
		$result = sqlInsert($query);

		if($result == null || $result['rows'] <=0 || $result['id'] <=0)
			error('Unable to create network! Unknown system problem.');
		
		$networkid = $result['id'];

		/* If this network is not a root-level network, copy user
		 permisssions of parent (from User2Network)  */
 		
		if(parentid > 0)
		{	
			/* Hmm... maybe we wont do this right now.  Not sure if this 
				is the best behavior. */
		}
	
		/* Return this new networks's netorkd id. */
		echo tag('result', tag('networkid', $networkid));
		
	}


	/****************************************************
	* Change a network now.  Sigh.  This is boring.
	*
	* "We were never being boring 'cause we were never being bored."
	*   -- Pet Shop Boys, circa 1990
	*
	**/
	function modifyNetwork($networkid, $name, $description, $ip, $mask, $locationLat, 
								  $locationLon, $lostHeartbeatCount, $parentid, $networkType,
								  $contactName,
								  $contactPhone,
								  $contactEmail,
								  $contactOtherInfo,
								  $notifyContact)
	{
		global $currentUserId;
		global $NETWORK_LEN;

		/* Make sure the current user is allowed to modify this network */
	
		if(!canAccessNetwork($currentUserId, $networkid))
			error("Unauthorized access to this network.");

		$parentid = (int) $parentid;
		$lostHeartbeatCount = (int) $lostHeartbeatCount;

		if(strlen($name) <1)
			error("Please specify a network name.");
	
		$query =   " UPDATE Networks "
					. " SET name =    '" . cleanTrimParam($name, $NETWORK_LEN["name"]) 					. "', "
					. " description = '" . cleanTrimParam($description, $NETWORK_LEN["description"])  . "', "
					. " parentid  = $parentid, "
					. " changedBy = $currentUserId, "
					. " centerLat =   '" . cleanTrimParam($locationLat, $NETWORK_LEN["locationLat"]) . "', "
					. " centerLon =   '" . cleanTrimParam($locationLon, $NETWORK_LEN["locationLon"]) . "', "
					. " lostHeartbeatThreshold = $lostHeartbeatCount, "
					. " ip =      '" 	. cleanTrimParam($ip,   $NETWORK_LEN["ip"])   . "', "
					. " netmask = '"  . cleanTrimParam($mask, $NETWORK_LEN["mask"]) . "', "
					. " type =    '" . cleanTrimParam($networkType, $NETWORK_LEN["type"]) . "', "
					. " contactName = '" . cleanTrimParam($contactName, $NETWORK_LEN["contactName"]) . "', "
					. " contactPhone = '" . cleanTrimParam($contactPhone, $NETWORK_LEN["contactPhone"]) . "', "
					. " contactEmail = '" . cleanTrimParam($contactEmail, $NETWORK_LEN["contactEmail"]) . "', "
					. " contactOtherInfo = '" . cleanTrimParam($contactOtherInfo, $NETWORK_LEN["contactOtherInfo"]) . "', "
					. " notifyContact = " . $notifyContact
					. " WHERE id = $networkid ";


		sqlUpdate($query);

		echo tag('result','success');
	}



	/****************************************************
	* Remove a network - dont let remove default networks.
	*
	**/
	function deleteNetwork($networkId)
	{ 
		$networkId = (int)$networkId;

		global $currentUserId;
		
		/* Make sure the current user is allowed to modify this network */
	
		if(!canAccessNetwork($currentUserId, $networkId))
			error("Unauthorized access to this network.");

		$query = "SELECT id FROM Networks WHERE isdefault > 0" ;

		$defaultNetworkId  = sqlNumber($query, 'id');

		if($defaultNetworkId == $networkId){
			error("You cannot remove the default network.");
			return;
		}

		/* Set parent networks to have root parent id */
		sqlCmd( " UPDATE Networks SET parentId = 0 "
				. " WHERE parentId = $networkId");

		/* Set hosts to the default network */
		sqlCmd(
				" UPDATE Hosts  "
			.  " SET networkId = " . $defaultNetworkId 
			.  " WHERE networkId = " . $networkId);

		/* Remove network permissions entries */
		sqlCmd(
				" DELETE FROM User2Network "
			. 	" WHERE networkid = " . $networkId);
		
		/* Update custom registration rules */
		sqlCmd( " Update CustomRegistrationRules "
				. " SET networkId = NULL WHERE "
				. " networkId = $networkId ");
 
		/* Farewell network */
		sqlCmd(" DELETE From Networks WHERE id = " . $networkId);
		
		/* And that should be it! */

		echo tag('result','success');

	}

	/*********************************************************************
	* Get networks for display in the manage Networks table
	* This is a flat list.
	**/
	function getManageNetworksXML()
	{
		global $currentUserId; 

		$depth = -1;
		$isrestricted = isUserNetworkUser($currentUserId);
		$networkids = array();
		
		if($isrestricted)
			$networkids = getAllowableNetworkIds($currentUserId);

		$networks = getManageNetworkChildren(0, $isrestricted, $currentUserId, 
														 $networkids, $depth);

		return "<networks>" . $networks . "</networks>";
		
	}


	/*********************************************************************
	*  Get nested children of flat network list.
	*
	**/
	function getManageNetworkChildren($networkid, $isrestricted, $userid, 
												 $networkids, $depth)
	{
		$depth++;

		$conn = getConnection();

		$query =    " SELECT Nets.*, "

					// Host counts:	
					.  "    (SELECT COUNT(id) FROM  Hosts "
					. 	"     WHERE Hosts.networkid = Nets.id ) as deviceCount, "
					
					.  "    (SELECT COUNT(*) FROM  Hosts "
					. 	"     WHERE Hosts.networkId = Nets.id "
					.  "     AND   Hosts.isdown > 0 ) as deviceDownCount,"
					
					.  "    (SELECT Count(Hosts.id) "
					.  "      FROM Hosts, AlarmProfile2AlarmDef, Alarms "
					. 	" 		Left Join ActiveAlarm on Alarms.id = ActiveAlarm.alarm_id "
					.  "     WHERE  "
					.  "     Hosts.networkid = Nets.id "
					. 	"		AND ( ActiveAlarm.alarm_id is not null) "
					.  "   	AND Alarms.cleared = 0 "
					.  "   	AND Alarms.hostId = Hosts.id "
					.  "   	AND Alarms.alarmId = AlarmProfile2AlarmDef.alarmId "
					.  "   	AND AlarmProfile2AlarmDef.alarmProfileId = Hosts.alarmProfileId  "
					.  "   	AND AlarmProfile2AlarmDef.severity = 'critical' "
					. 	"     ) as criticalAlerts, "

					.  "    (SELECT Count(Hosts.id) "
					.  "      FROM Hosts,AlarmProfile2AlarmDef, Alarms  "
					. 	" 		Left Join ActiveAlarm on Alarms.id = ActiveAlarm.alarm_id "
					.  "     WHERE  "
					.  "     Hosts.networkid = Nets.id "
					. 	"		AND (ActiveAlarm.alarm_id is not null) "
					.  "   	AND Alarms.cleared = 0 "
					.  "   	AND Alarms.hostId = Hosts.id "
					.  "   	AND Alarms.alarmId = AlarmProfile2AlarmDef.alarmId "
					.  "   	AND AlarmProfile2AlarmDef.alarmProfileId = Hosts.alarmProfileId  "
					.  "   	AND AlarmProfile2AlarmDef.severity = 'warning' "
					. 	"     ) as warningAlerts, "

					// Now see if we have any uncleared alerts at all
					.  "    (SELECT Count(Hosts.id) "
					.  "      FROM Hosts, Alarms "
					.  "     WHERE  "
					.  "     Hosts.networkid = Nets.id "
					.  "   	AND Alarms.cleared = 0 "
					.  "   	AND Alarms.hostId = Hosts.id "
					. 	"     ) as unclearedAlerts "

				   .  " FROM Networks Nets "
					.  " WHERE Nets.parentid = $networkid "
					.  " ORDER BY Nets.name";

		$result = @mysql_query($query, $conn);

		if(!$result)
			error("Unable to retrieve networks!" . mysql_error());

		$numrows = @mysql_numrows($result);

		$retVal = "";

		$i = 0;

		while($i < $numrows)
		{	
			$add = TRUE;

			$network = mysql_fetch_assoc($result);

			if($isrestricted)
			{
				$add = FALSE;
				if(in_array($network['id'], $networkids)) 
					$add = TRUE;
			}	

			if($add)
			{


				if(is_numeric($network['centerLon']) && is_numeric($network['centerLat']))
					$valid = 1;
				else $valid = 0;
				
				$notifyContact = $network['notifyContact'];
				if ($notifyContact != '1') {
				  $notifyContact = '0';
				}
				
				$retVal .= "<network "
							. " parentid='"   . $network['parentid'] 			. "' "
							. " heartbeat='"  . $network['lostHeartbeatThreshold'] . "' "
							. " depth='$depth' "
							. " devices='"    . $network['deviceCount'] 		. "' "
							. " devicesDown='". $network['deviceDownCount'] . "' "
							. " criticalAlerts='" . $network['criticalAlerts'] 	. "' "
							. " warningAlerts='"  . $network['warningAlerts'] . "' "
							. " unclearedAlerts='" . $network['unclearedAlerts'] . "' "
							. " isdefault='" 	. $network['isdefault'] 		. "' "
							. " id='"   		. $network['id']     			. "' "
							. " netmask='"		. $network['netmask']   		. "' " 
							. " valid='$valid' "
							. ">"
							. tag('name',$network['name'], true) 
							. tag('desc',$network['description'], true) 
							. tag('ip',$network['ip'], true) 
							. tag('id',$network['id']) 
							. tag('type',$network['type']) 
							. tag('centerlon',$network['centerLon'], true) 
							. tag('centerlat',$network['centerLat'], true)
							. tag('contactName', $network['contactName'], true)
							. tag('contactPhone', $network['contactPhone'], true)
							. tag('contactEmail', $network['contactEmail'], true)
							. tag('contactOtherInfo', $network['contactOtherInfo'], true)
							. tag('notifyContact', $notifyContact)
							."</network>\n";
			}
			$retVal .= getManageNetworkChildren($network['id'], $isrestricted, $userid, $networkids, $depth);

	

			$i++;
		}

		return $retVal;
	}

	
}


?>
