<?php

include('lib/sql.php');
include('lib/host.php');

include('lib/settings-utils.php');
include('lib/ipaddr-utils.php');
include('lib/verify-utils.php');


if(!function_exists('task_wrapper'))
{

  function task_wrapper() {}


  /* Lengths of DB fields in the Tasks table */
  $TASKS_LEN = array(
        'name'      => 32);

  
  function handleTaskAction($action)
  {
    if($action == null)
      error('Unauthorized');


    switch($action)
    {
      case 'create':
        $task      = req('task');

        /* Create a firmware download task with a managed fw file. */
        if($task == 'firmware-download-managed')
        {
          echo tag('result', tag('taskid', createManagedFwDownloadTask(req('name') , req('now'), 
                        req('date'),  req('hosts'), 
                        req('firmwareId'), req('https'))));
          return;
        }
        
        /* Create a firmware download task with a url. */
        if($task == 'firmware-download-url')
        {
          echo createURLFwDownloadTask(req('name') , req('now'), req('date'), 
                                req('hosts'), req('url'));
          return;
        }

        /* Create a firmware download task via file upload */
        if($task == 'firmware-download-upload')
        {
          echo createUploadFwDownloadTask(req('https'));
          return;

        }

        /* This is firmware download + upgrade operation, part 2 */
        if($task == 'firmware-operation-upload') 
        {
          echo createUpgradeTaskFromDownload(req('downloadTaskId'), 
                      req('downloadName'), req('upgradeName'),
                      req('downloadNow'), req('downloadDate'),
                      req('upgradeNow'), req('upgradeDate'),
                      req('hosts'));
          return;

        }
      
        if($task == 'firmware-operation-managed')
        {
          echo createManagedFwOperationTask(req('firmwareId'), 
                      req('downloadName'), req('upgradeName'),
                      req('downloadNow'), req('downloadDate'),
                      req('upgradeNow'), req('upgradeDate'),
                      req('hosts'), req('https'));
          return;

        }

        if($task == 'firmware-operation-url')
        {
          echo createURLFwOperationTask(req('url'), 
                      req('downloadName'), req('upgradeName'),
                      req('downloadNow'), req('downloadDate'),
                      req('upgradeNow'), req('upgradeDate'),
                      req('hosts'));
          return;

        }

        /* Create a config download task from a managed file */
        if($task == 'config-download-managed')
        {
          echo createManagedConfigDownloadTask(req('configId'), req('name'),
                  req('now'), req('date'), req('hosts'), req('reboot'));
          return;
        }

        /* Create a config download task using the given text as the config */
        if($task == 'config-download-text')
        {
          echo createTextConfigDownloadTask( req('name'), req('hosts'), req('reboot'),
                              req('config'), req('now'), req('date'), req('cfgId'));
          return;

        }

        /* Create a config download task by uploading a config file */
        if($task == 'config-download-upload')
        {
      
          echo createUploadConfigDownloadTask();
          return;
        }

        /* This is creating a license-download task by uploading a file */
        if($task == 'licence-download-upload')
        {

          echo createUploadLicenseDownloadTask();
          return;
        }

        
        /* Else we're creating a basic task. */
        createTask($task, req('name') , req('now'), req('date'),  req('hosts'));
      break;
        
      case 'delete':
        echo  removeTask(req('id'));
      break;

      case 'clear':
        echo  clearTask(req('id'));
      break;

      case 'clearSelected':
        echo  clearSelectedTask(req('taskIds'));
      break;

      case 'deleteHost':
        echo removeHostFromTask(req('hostid'), req('taskid'));
      break;

      case 'detail':
        echo tag('result', getTaskDetail(req('id')));
      break;
      
      case 'update':
        echo updateTask(req('id'), req('name'), req('now'), req('date'), 
                        req('hosts'));
      break;

      case 'update-fw':
        echo updateFwURLTask(req('id'), req('name'), req('url'), req('date'), 
                        req('hosts'));
      break;
      case 'get':
        echo tag('result',  getTaskList());
      break;

      case 'clearCompleted':
        echo tag('result', clearCompletedTasks());

      break;
      case 'fetch':
      
      $type = req('type');
      if($type == 'license-input')
        fetchLicenceInput(req('id'));
      else if($type == 'config-input')
        fetchConfigInput(req('id'));
      else if($type == 'license-output')
         fetchLicenseOutput(req('hostId'));
      else if($type == 'license-output-contents')
         fetchLicenseOutputContents(req('hostId'));
      else if($type == 'config-output')
         fetchConfigOutput(req('taskId'), req('hostId'));
      else if($type == 'config-output-contents')
         fetchConfigOutputContents(req('taskId'), req('hostId'));
      else if($type == 'trouble-output')
        fetchTroubleFile(req('taskId'), req('hostId'));
      else if($type == 'trouble-output-contents')
        fetchTroubleFileContents(req('id'), req('hostId'));
      else if($type == 'config-input-contents')
        fetchConfigInputContents(req('id'));

      break;
    }

  }


  /***************************************************
  * Get task details
  */
  function getTaskDetail($taskId)
  {

    $taskId = (int) $taskId;

    $query = " SELECT t.name, t.type, t.id, t.createdOn as submittedOn, "
      .  " Users.username as createdBy, "
      .  " CASE WHEN t.type like 'firmware-download' THEN t.text_data ELSE '' END as url, "
      .  " CASE WHEN t.binary_data is not null THEN 1 ELSE 0 END as hasBinaryData, "        
      .  " (SELECT dateExecute from Host2Task WHERE Host2Task.idtask = t.id LIMIT 0,1) as executeDate, "
      .  " (SELECT Count(idhost) from Host2Task WHERE Host2Task.idtask = t.id) AS totalDevices, "
      .  " (SELECT Count(idhost) from Host2Task WHERE Host2Task.idtask = t.id AND Host2Task.status like 'ok') AS okayDevices, "
      .  " (SELECT Count(idhost) from Host2Task WHERE Host2Task.idtask = t.id AND Host2Task.status is NULL) AS incompleteDevices,  " 
      .  " (SELECT Count(idhost) from Host2Task WHERE Host2Task.idtask = t.id AND Host2Task.status like 'failed') AS errorDevices,  "
      .  " (SELECT Count(idhost) from Host2Task WHERE Host2Task.idtask = t.id AND Host2Task.dateCompleted is NULL) AS pendingDevices  " 
      .  " FROM Task t left join Users "
      .  " on t.createdby = Users.id "
      .  " WHERE t.id = $taskId ";
     
    $retVal = "<details>";

    $task = @sqlRow($query);

    if(!$task)
      error("Unable to retrieve task details: " . mysql_error());

    foreach($task as $key=>$value)
    {
      $detailTag = tag($key, $value);
      $retVal = $retVal . $detailTag;
    }

    $retVal .= "</details>";

    $retVal .= "<devices>";


    $query = " SELECT idhost as id, ip, idtask, name, dateCompleted, status, message, "
         . " CfgBackup.cfg is not null AS hasConfig, "
         . " license is not null as hasLicense, "
         . " TroubleshootFile.data is not null as hasTroubleFile "
         . " FROM  "
         . " ( SELECT Hosts.id as idhost, Host2Task.idtask, Hosts.name, "
         . "   Host2Task.dateCompleted, Host2Task.status, Host2Task.message, "
         . "   Hosts.license, INET_NTOA(Hosts.ipaddr) as ip "
         . "   FROM Hosts , Host2Task  "
         . "   WHERE Host2Task.idtask = $taskId  "
         . "   AND Host2Task.idhost = Hosts.id) as A  "
         . " Left Join CfgBackup  "
         . " On CfgBackup.hostid  = A.idhost  "
         . " AND CfgBackup.taskid = A.idtask "
         . " Left Join TroubleshootFile "
         . " On TroubleshootFile.taskid = A.idtask "
         . " AND TroubleshootFile.hostid = A.idhost ";
    
    $conn = getConnection();


    $result = @mysql_query($query, $conn);

    if(!$result)
    {
      error("Unable to retrieve task's device list! ". mysql_error());
      return;
    }

    $numrows = mysql_numrows($result);

    $i = 0;

    while($i < $numrows)
    { 
      $host = mysql_fetch_assoc($result);
      
      $retVal .= "<host>\n";

      foreach($host as $key=>$value)
      {     
        $hostTag = tag($key, $value);

        $retVal .= $hostTag;
      }

      $retVal .= "</host>";
      $i++;
    }

    $retVal .= "</devices>";
    
    return $retVal;
  }

  /***************************************************
  * Get a list of tasks (uncleared)
  */
  function getTaskList()
  {

    global $currentUserId;

    // Need to filter out tasks that aren't owned by this user

    $isNetworkUser = isUserNetworkUser($currentUserId);
    
    $networkFilter = "";

    if($isNetworkUser)
    {
      $networkFilter = " AND (0 ";
      $allowedNetworkIds = getAllowableNetworkIds($currentUserId);

      foreach ($allowedNetworkIds as $networkId)
      {
        $networkId = (int) $networkId;      

        $networkFilter .= " OR Hosts.networkId = $networkId ";

      }
      $networkFilter .= ")";
    }

    $query = " SELECT t.name, t.type, t.id, t.createdon as submittedOn, Users.username as createdBy, "
        .  " Now() as serverTime, "
        .  " (SELECT dateExecute from Host2Task WHERE Host2Task.idtask = t.id LIMIT 0,1) as executeDate, "
        .  " (SELECT Count(idhost) from Host2Task WHERE Host2Task.idtask = t.id AND Host2Task.status like 'ok') AS okayDevices, "
        .  " (SELECT Count(idhost) from Host2Task WHERE Host2Task.idtask = t.id AND Host2Task.status is NULL) AS incompleteDevices,  " 
        .  " (SELECT Count(idhost) from Host2Task WHERE Host2Task.idtask = t.id AND Host2Task.status like 'failed') AS errorDevices,   "
        .  " (SELECT Count(idhost) from Host2Task WHERE Host2Task.idtask = t.id AND Host2Task.dateCompleted is NULL) AS pendingDevices,  " 
        .  " (SELECT Count(idhost) from Host2Task WHERE Host2Task.idtask = t.id) AS totalDevices "
        .   ($isNetworkUser? " , (Select Count(idhost) From Host2Task, Hosts "
                      . "  WHERE Hosts.id = Host2Task.idhost "
                      .  " AND Host2Task.idtask = t.id "
                      . " $networkFilter) AS totalInNetwork " : "")
        .  " FROM Task t left join Users "
        .  " on t.createdby = Users.id "
        .  " WHERE cleared = 0 ";
  

    $conn = getConnection();
    $result = @mysql_query($query, $conn);

    if(!$result)
      error("Unable to retrieve task list! " . mysql_error());

    $numrows = mysql_numrows($result);

    $retVal = "<tasks>";

    $i = 0;

    while($i < $numrows)
    { 
      $task = mysql_fetch_assoc($result);
      
      if($isNetworkUser)
      {
        // If not all devices are in our allowed networks then skip.
        $total = (int)$task["totalDevices"];
        $totalInNetwork= (int)$task["totalInNetwork"];
        if($total == 0 || ($total > $totalInNetwork)){
          $i++;
          continue;
        }
      }
      $retVal .= "<task>\n";

      foreach($task as $key=>$value)
      {     
        $tag = tag($key, $value);

        $retVal .= $tag;
      }

      $retVal .= "</task>";
      $i++;
    }
    $retVal .= "</tasks>";

    return $retVal; 
    
  }

  /**************************************************************************
  * Update a firmware download task (method via manual URL) task
  *
  */
  function updateFwURLTask($taskId, $name, $url, $date, $hosts)
  {

    global $TASKS_LEN;

    $id   = (int) $id;

    $name = cleanTrimParam($name, $TASKS_LEN['name']);

    $query = "UPDATE Task "
        . " SET name = '$name' , url = '$url' "
        . " WHERE id = $id ";

    @sqlUpdate($query);

    $error = mysql_error();

    if($error != "")
      error($error);

    updateHostsToTask($taskId, 0, $date, $hosts);

    return tag('result', 'success');
     

  }

  /**************************************************************************
  * Update a basic task
  *
  */
  function updateTask($id, $name, $now, $date, $hosts)
  {
  
    global $TASKS_LEN;

    $id   = (int) $id;

    $name = cleanTrimParam($name, $TASKS_LEN['name']);

    $query = "UPDATE Task "
        . " SET name = '$name' "
        . " WHERE id = $id ";


    @sqlUpdate($query);

    $error = mysql_error();

    if($error != "")
      error($error);

    updateHostsToTask($id, $now, $date, $hosts);



    return tag('result', 'success');
     

  }

  /**************************************************************************
  * Creat a new basic task
  *
  *  $timeOffsetInMin = time in minutes to add to the task execuite time (ONLY applies
  *  if the task is set to run 'NOW'.)  This is to avoid doing time calculations 
  *  in php and keep in sql.
  */
  function createTask($type, $name, $now, $executeDate, $hosts, $timeOffsetInMin = 0)
  { 
    global $currentUserId;
    global $TASKS_LEN;

    $name = cleanTrimParam($name, $TASKS_LEN['name']);

    $guid = rand_str();

    /* 1. Create the task.  */
    /* Add something in the text_data field so agent won't ignore it ... */
    $query = "INSERT INTO Task "
        . " (name, type, createdBy, guid, text_data) "
        . " VALUES "
        . " ('$name', '$type', $currentUserId, '$guid', 1) ";

     
    $result = @sqlInsert($query);

    if($result == null || $result['rows'] <=0 || $result['id'] <=0)
      error('Unable to create task! ' . mysql_error());
    
    $taskId = $result['id'];
    
    /* 2.  Add hosts entries */
    addHostsToTask($taskId, $now, $executeDate, $hosts, 0,  $timeOffsetInMin);
     
    return tag("result", tag("taskid", $taskId));
  
  }

  /**********************************************************************
  * A task was updated.  
   * - Let's remove all hosts that haven't run yet then add them back.  
  */
  function updateHostsToTask($taskId, $now, $date, $hosts)
  {
    $query = " DELETE FROM Host2Task "
        .  " WHERE idtask = $taskId "
        .  " AND (message is null or message like '') "
        .  " AND dateCompleted is null ";

    @sqlUpdate($query);
    
    return addHostsToTask($taskId, $now, $date, $hosts);
  }

  /**********************************************************************
  * Add the entires in Host2Task for the given task and hosts.
  *
  */
  function addHostsToTask($taskId, $now, $date, $hosts, $ignoreErrors = TRUE, $timeOffsetInMins=0)
  {
    $taskId = (int) $taskId;


    if($now > 0)
      $date  = "NOW() ";

    else $date = "'" . $date ."'";

    $date = " Date_Add( $date, Interval $timeOffsetInMins MINUTE) ";

    $pieces = explode("|", $hosts);

    if(is_array($pieces) && count($pieces) > 0)
    { 
      foreach($pieces as $hostId)
      {
        if(!is_numeric($hostId))
          continue;

        $query = "INSERT INTO Host2Task "
            . " (idhost, idtask, dateExecute, dateSubmitted) "
            . " VALUES "
            . " ($hostId, $taskId, $date, NOW() ) ";

        //error($query);
        // Ignore duplicate entries... 
        @sqlUpdate($query); 
        if(!$ignoreErrors)
        {   
          $error = mysql_error();
          if($error != "")  error($error);
        }
      }
    }
    else error("Invalid list of hosts!");
  }

   /**************************************************************
  * This is for creating a fw download + upgrade in one step 
  * where the config file is from a manual URL
  *
  */
  function createURLFwOperationTask($url, $downloadTaskName, 
                      $upgradeTaskName, $downloadNow, $downloadDate,
                      $upgradeNow,  $upgradeDate, $hosts)
  {
    /* Create download task */
    createURLFwDownloadTask($downloadTaskName, $downloadNow, $downloadDate, 
                    $hosts, $url);

    /* Create upgrade task */
    return createTask('firmware-upgrade', $upgradeTaskName , $upgradeNow, 
                $upgradeDate,  $hosts);

  }

   /**************************************************************
  * This is for creating a fw download + upgrade in one step 
  * where the config file is a managed config file.
  *
  */
  function createManagedFwOperationTask($firmwareId, $downloadTaskName, 
                      $upgradeTaskName, $downloadNow, $downloadDate,
                      $upgradeNow,  $upgradeDate, $hosts, $https)
  {
    /* 1. Create the firmware download task */
  
    $taskId = createManagedFwDownloadTask($downloadTaskName, $downloadNow, 
                              $downloadDate, $hosts, $firmwareId, 
                              $https);

    if($taskId <=0)
      error("Unable to create task!");

    /* 2. Create the firmware upgrade task */
    
    return createTask('firmware-upgrade', $upgradeTaskName , $upgradeNow, 
                $upgradeDate,  $hosts);
  }

   /**************************************************************
  * This is for creating a fw download + upgrade in one step.
   * This assumes the download task has already been created. )
  *
  */
  function createUpgradeTaskFromDownload($downloadTaskId, $downloadTaskName, 
                      $upgradeTaskName, $downloadNow, $downloadDate,
                      $upgradeNow,  $upgradeDate, $hosts)
  {
    global $TASKS_LEN;

    $downloadTaskId = (int) $downloadTaskId;
    $downloadNow    = (int) $downloadNow;
    $upgradeNow    = (int) $upgradeNow;

    $upgradeTaskName = cleanTrimParam($upgradeTaskName, $TASKS_LEN['name']);
    $downloadTaskName = cleanTrimParam($downloadTaskName, $TASKS_LEN['name']);
    
    /* 1. Update our download task with the other values */
    updateTask($downloadTaskId, $downloadTaskName, $downloadNow, $downloadDate, 
            $hosts);

    /* 2. Create our upgrade task */
    return createTask('firmware-upgrade', $upgradeTaskName , $upgradeNow, 
                $upgradeDate,  $hosts);

  }


   /**************************************************************
  * Create a config download task (method: upload a file)
  *
  */
  function createUploadConfigDownloadTask()
  {
    global $currentUserId;

    
    if($_FILES['Filedata']['size'] <= 0)
    {
      error("Unable to upload file!");
    }

    $type     = 'config-download';

    $tmpName  = $_FILES['Filedata']['tmp_name'];
    $fileName = $_FILES['Filedata']['name'];

    $fp = fopen($tmpName, 'r');
    $content = fread($fp, filesize($tmpName));
    fclose($fp);

    $content = cleanParam($content);

    $guid = rand_str();

    /* 1. Create the task.  */

    $query = "INSERT INTO Task "
        . " (name, type, createdBy, guid, text_data) "
        . " VALUES "
        . " ('[Temp Config Download task]', '$type', $currentUserId, "
        . "  '$guid',  '$content') ";

     
    $result = @sqlInsert($query);

    if($result == null || $result['rows'] <=0 || $result['id'] <=0)
      error('Unable to create task! ' . mysql_error());
    
    $taskId = $result['id'];


    return tag('result', tag('taskid', $taskId));

  }


    /**************************************************************
  * Create a license download task (method: upload a file)
  *
  */
  function createUploadLicenseDownloadTask()
  {
    global $currentUserId;

    
    if($_FILES['Filedata']['size'] <= 0)
    {
      error("Unable to upload file!");
    }

    $type     = 'licence-download';

    $tmpName  = $_FILES['Filedata']['tmp_name'];
    $fileName = $_FILES['Filedata']['name'];

    $fp = fopen($tmpName, 'r');
    $content = fread($fp, filesize($tmpName));
    fclose($fp);

    $content = cleanParam($content);

    $guid = rand_str();

    /* 1. Create the task.  */

    $query = "INSERT INTO Task "
        . " (name, type, createdBy, guid, text_data) "
        . " VALUES "
        . " ('[Temp Config Download task]', '$type', $currentUserId, "
        . "  '$guid',  '$content') ";

     
    $result = @sqlInsert($query);

    if($result == null || $result['rows'] <=0 || $result['id'] <=0)
      error('Unable to create task! ' . mysql_error());
    
    $taskId = $result['id'];

    return tag('result', tag('taskid', $taskId));

  }

   /**************************************************************
  * Create a fw download task (method: upload a file)
  *
  */
  function createUploadFwDownloadTask($https)
  {
    global $currentUserId;

    
    if($_FILES['Filedata']['size'] <= 0)
    {
      error("Unable to upload file!");
    }

    $type     = 'firmware-download';

    $tmpName  = $_FILES['Filedata']['tmp_name'];
    $fileName = $_FILES['Filedata']['name'];

    $fp = fopen($tmpName, 'r');
    $content = fread($fp, filesize($tmpName));

    fclose($fp);

    $content = cleanParam($content);

    $guid = rand_str();
    $url  = generateURL($guid, $https);

    if ($url === FALSE) {
      /*
       * TBD: define the markup to return on error, make sure the GUI
       * recognizes and handles it.
       */
       return '<result />';
    }
    
    $url  = cleanParam($url);

    /* 1. Create the task.  */

    $query = "INSERT INTO Task "
        . " (name, type, createdBy, guid, text_data, binary_data) "
        . " VALUES "
        . " ('Temp FW task', '$type', $currentUserId, '$guid', '$url', '$content') ";

     
    $result = @sqlInsert($query);

    if($result == null || $result['rows'] <=0 || $result['id'] <=0)
      error('Unable to create task! ' . mysql_error());
    
    $taskId = $result['id'];

    return tag('result', tag('taskid', $taskId));

  }


   /**************************************************************
  * Create a fw download task (method: use manual URL)
  *
  */
  function createURLFwDownloadTask($name, $now, $date, $hosts, $url)
  {
    global $TASKS_LEN;
    global $currentUserId;

    $name     = cleanTrimParam($name, $TASKS_LEN['name']);
    $firmwareId = (int) $firmwareId;
    $url       = cleanParam($url);

    if($url == "")
      error("Please enter a URL.");

    $guid = rand_str();
    $type = "firmware-download";

    /* 1. Create the task.  */

    $query = "INSERT INTO Task "
        . " (name, type, createdBy, guid, text_data) "
        . " VALUES "
        . " ('$name', '$type', $currentUserId, '$guid', '$url') ";

     
    $result = @sqlInsert($query);

    if($result == null || $result['rows'] <=0 || $result['id'] <=0)
      error('Unable to create task! ' . mysql_error());
    
    $taskId = $result['id'];

    /* 2.  Add hosts entries */

    addHostsToTask($taskId, $now, $date, $hosts);
     
    return tag("result", tag("taskid", $taskId));
  }


  /**************************************************************
  * Create a config download task - this is called from the 
  * device details control, but more than one host can be added to the task.
  *
  */
  function createTextConfigDownloadTask($name, $hosts, $reboot, $config, $now,
                           $date, $cfgId)
  {
    global $TASKS_LEN;
    global $currentUserId;

    $reboot = (int) $reboot;
    $cfgId = (int) $cfgId;

    $name = cleanTrimParam($name, $TASKS_LEN['name']);
    $config  = cleanParam($config);

    $guid = rand_str();
    $type = "config-download";

    /* 1. Create the task.  */

    $query = "INSERT INTO Task "
        . " (name, type, createdBy, guid, text_data) "
        . " VALUES "
        . " ('$name', '$type', $currentUserId, '$guid', '$config') ";

     
    $result = @sqlInsert($query);

    if($result == null || $result['rows'] <=0 || $result['id'] <=0)
      error('Unable to create task! ' . mysql_error());
    
    $taskId = $result['id'];

  
    /* 2.  Add hosts entries */

    addHostsToTask($taskId, $now, $date, $hosts);
     
    /* 3. (optional) If reboot required, then create reboot task for all hosts that runs 2 mins after  */
    if($reboot)
    {
      
      $max = getMaxHB($hosts);

      $max = $max/60; // get minutes

      $max += 1;
      
      createTask('reboot', $name . " (Reboot)" , $now, $date,  req('hosts'), $max);
    }

    /* 4. (optional) If cfg is specified, then update CfgBackup with this task id. */
    if($cfgId > 0)
    {
      $query = "UPDATE CfgBackup SET taskId = $taskId WHERE id = $cfgId ";
      sqlUpdate($query);
    }
    return tag("result", tag("taskid", $taskId));

  }

  /**************************************************************
  * Create a config download task (method: use DB managed file)
  *
  */
  function createManagedConfigDownloadTask($configId, $name, $now, $date, 
             $hosts, $reboot)
  {
    global $TASKS_LEN;
    global $currentUserId;

    $reboot = (int) $reboot;

    $name = cleanTrimParam($name, $TASKS_LEN['name']);
    $configId = (int) $configId;
    $now = (int) $now;

    if($configId <=0)
      error("Please select a Config File.");

    $guid = rand_str();
    $type = "config-download";

    /* 1. Create the task.  */

    $query = "INSERT INTO Task "
        . " (name, type, createdBy, guid) "
        . " VALUES "
        . " ('$name', '$type', $currentUserId, '$guid') ";

     
    $result = @sqlInsert($query);

    if($result == null || $result['rows'] <=0 || $result['id'] <=0)
      error('Unable to create task! ' . mysql_error());
    
    $taskId = $result['id'];

    /* 2. Copy over config file */

    $query = "UPDATE Task SET text_data = "
        . "    (SELECT ConfigFiles.cfg from ConfigFiles "
        . "     WHERE ConfigFiles.id = $configId) "
        . " WHERE Task.id = $taskId";

    @sqlUpdate($query);

    /* 3.  Add hosts entries */

    addHostsToTask($taskId, $now, $date, $hosts);
     
    /* 4. (optional) If reboot required, then create reboot task for all hosts that runs 2 mins after  */
    if($reboot)
    {

      /* Add max host's HB interval to the date  */

      $max = getMaxHB($hosts);

      $max = $max/60; // get minutes

      
      createTask('reboot', $name . " (Reboot)" , $now, req('date'),  req('hosts'), $max);
    }

    return tag("result", tag("taskid", $taskId));

  }

  

  /**************************************************************
  * Create a fw download task (method: use DB managed file)
  *
  */
  function createManagedFwDownloadTask($name, $now, $date, $hosts,  $firmwareId, $https)
  {
    global $TASKS_LEN;
    global $currentUserId;

    $name     = cleanTrimParam($name, $TASKS_LEN['name']);
    $firmwareId = (int) $firmwareId;

    if($firmwareId <=0)
      error("Please select a Firmware File.");

    $guid = rand_str();
    $type = "firmware-download";
    $url  = generateURL($guid, $https);
    
    if ($url == FALSE) {
      /* TBD: make sure the calling code handles this return value */
      return -1;
    }
    
    $url = cleanParam($url);

    /* 1. Create the task.  */

    $query = "INSERT INTO Task "
        . " (name, type, createdBy, guid, text_data) "
        . " VALUES "
        . " ('$name', '$type', $currentUserId, '$guid', '$url') ";

     
    $result = @sqlInsert($query);

    if($result == null || $result['rows'] <=0 || $result['id'] <=0)
      error('Unable to create task! ' . mysql_error());
    
    $taskId = $result['id'];

    /* 2. Copy over file */

    $contents = sqlString("SELECT FirmwareFiles.contents from FirmwareFiles "
        . "     WHERE FirmwareFiles.id = $firmwareId", "contents");

    $contents = cleanParam($contents);

    $query = "UPDATE Task SET binary_data = '$contents' "
        . " WHERE Task.id = $taskId";

    @sqlUpdate($query);

    /* 3.  Add hosts entries */

    addHostsToTask($taskId, $now, $date, $hosts);
     
    return $taskId;
  }


  function makeBaseURL($addr, $https)
  {
    if ($addr === '' || $addr === FALSE) {
      return FALSE;
    }
    
    return $https ? "https://" . $addr : "http://" . $addr;
  }

  /**************************************************************
  * Create a url for a fw download task
  *
  */
  function generateURL($taskGuid, $https)
  {
  
    /* Get the server address from the system settings */
    
    $url = makeBaseURL(getServerAddressSetting(), $https);
    
    /* 
     * If there is no URL set in the configuration, or it's not
     * right, try and autodetect the server address
     */
     
    if ($url === FALSE /* || !verifyServerURL($url) */) {
      
      $serverAddr = detectServerAddressAndPort(LOOKUP_WAN_ADDRESS);
      $url = makeBaseUrl($serverAddr, $https);
      
      if ($url === FALSE /* || !verifyServerURL($url) */) {

        $serverAddr = detectServerAddressAndPort(LOOKUP_LAN_ADDRESS);
        $url = makeBaseUrl($serverAddr, $https);
        
        if ($url === FALSE /* || !verifyServerURL($url) */) {
          $serverAddr = detectServerAddressAndPort();
          $url = makeBaseUrl($serverAddr, $https);
        }
      }
    }
    
    $self = $_SERVER['PHP_SELF'];
    $path = str_replace(basename($self), "", $self);
    
    $url .= $path . "getFirmware.php?taskId=" . $taskGuid;

    return $url;
  }


  /**************************************************************
  * Clear (aka hide or soft delete) a task from the database
  *
  */
  function clearTask($id)
  {
    $id = (int) $id;

    
    $query = "UPDATE Task SET cleared = 1 WHERE id = $id";

    @sqlCmd($query);

    $error = mysql_error();

    if($error != "")
      error($error);


    return tag("result", "success");
  }

  /*********************************************************
   * Clear the selected tasks.
   * $taskIdStr = id,id,id
   */
  function clearSelectedTask($taskIdStr)
  {
	  $query = "UPDATE Task SET cleared = 1 WHERE id in ($taskIdStr)";

	  @sqlCmd($query);

	  $error = mysql_error();

	  if($error != "")
	    error($error);

	  return tag("result", "success");
  }

  /**************************************************************
  * Remove an unrun task from the DB
  *
  */
  function removeTask($id)
  {
    // 1. Remove all entries in Host2Task
    // 2. Remove entry in Task

    $id = (int) $id;

    $query = "DELETE From Host2Task WHERE idtask = $id";

    @sqlCmd($query);

    $error = mysql_error();

    if($error != "")
      error($error);

    $query = "DELETE From Task WHERE id = $id";

    @sqlCmd($query);

    $error = mysql_error();

    if($error != "")
      error($error);


    return tag("result", "success");
  }

  /**************************************************************
  * Remove an unrun host from a task from the DB
  *
  */
  function removeHostFromTask($hostId, $taskId)
  {

    $hostId = (int) $hostId;
    $taskId = (int) $taskId;

    $query = "DELETE From Host2Task WHERE idtask = $id AND idhost = $idhost";

    @sqlCmd($query);

    $error = mysql_error();

    if($error != "")
      error($error);


    return tag("result", "success");
  }


  /**************************************************************
  * Set all tasks as 'cleared' that are completed
  *
  */
  function clearCompletedTasks()
  {
    $query = " Update Task Set Cleared = 1 "
         . " WHERE "  /* Where total devices = total completed devices */
         . " (SELECT count(idhost) FROM Host2Task WHERE Host2Task.idtask = id) " 
         . "  =  "
         . " (SELECT count(idhost) FROM Host2Task WHERE Host2Task.idtask = id "
         . "  AND dateCompleted is not null) ";
  
    @sqlCmd($query);

    $error = mysql_error();
    if($error != "")
      error("Unable to process request! $error");

    return 'success';
  }

  /**************************************************************
  * Get the license file from a task that was used as the input
  */
  function fetchLicenceInput($id)
  {
    $id = (int) $id;
    $query = "SELECT text_data FROM Task WHERE id = $id";

    $content = @sqlString($query, 'text_data');

    //$content = base64_decode($content);
    
    echo $content;
  }

  /**********************************************************************
  * Get the config file CONENTS from a task that was used as the input
  */
  function fetchConfigInputContents($id)
  {
    $id = (int) $id;
    $query = "SELECT text_data FROM Task WHERE id = $id";

    $content = @sqlString($query, 'text_data');

    echo tag('config', tag('contents', $content, true));
  }
  
  /**************************************************************
  * Get the config file from a task that was used as the input
  */
  function fetchConfigInput($id)
  {
    $id = (int) $id;
    $query = "SELECT text_data FROM Task WHERE id = $id";

    $content = @sqlString($query, 'text_data');

    echo $content;
  }

  function fetchTroubleFile($taskid, $hostid)
  {
    $id = (int) $id;
    $query = "SELECT data FROM TroubleshootFile "
        . " WHERE taskid = $taskid "
        . " AND hostid = $hostid";

    $content = @sqlString($query, 'data');

    // not doing error here since it writes it to the file
    if($content == null || $content == ''){
      echo("Unable to retrieve troubleshooting file. ");
      return;
    }
    
    echo $content;
  }

  /**********************************************
  * [OBSOLETE] - trouble file is a tar now.
  * 
  */
  function fetchTroubleFileContents($taskid, $hostid)
  {
    $id = (int) $id;
    $query = "SELECT data FROM TroubleshootFile "
        . " WHERE taskid = $taskid "
        . " AND hostid = $hostid";

    $content = @sqlString($query, 'data');

    echo tag( 'result', tag('contents', $content, true));
  }

  /**************************************************************
  * Get the host's resulting config file from the task
  */
  function fetchConfigOutput($taskid, $hostid)
  {
    $id = (int) $id;
    $query = "SELECT cfg  FROM CfgBackup "
        . " WHERE taskid = $taskid "
        . " AND hostid = $hostid";


    $content = @sqlString($query, 'cfg');

    // not doing error here since it writes it to the file
    if($content == null || $content == ''){
      echo("Unable to retrieve config file.");
      return;
    }

    echo $content;
  }

  /**************************************************************
  * Get the host's resulting config file contents from the task
  */
  function fetchConfigOutputContents($taskid, $hostid)
  {
    $taskid = (int) $taskid;
    $hostid = (int) $hostid;
    $query = "SELECT cfg  FROM CfgBackup "
        . " WHERE taskid = $taskid "
        . " AND hostid = $hostid";


    $content = @sqlString($query, 'cfg');

    // not doing error here since it writes it to the file
    if($content == null || $content == ''){
      echo("Unable to retrieve config file.");
      return;
    }

    echo tag('result', tag('contents', $content, true));
  }

  /**************************************************************
  * Get the host's resulting license file from the task
  */
  function fetchLicenseOutput($hostid)
  {
    $hostid = (int) $hostid;

    $query = "SELECT license  FROM Hosts "
        . " WHERE id = $hostid";

    $content = @sqlString($query, 'license');

    // not doing error()  here since it writes it to the file
    if($content == null || $content == ''){
      echo("Unable to retrieve license file.");
      return;
    }
    
    echo $content;
  }

  /**************************************************************
  * Get the host's resulting license file from the task
  */
  function fetchLicenseOutputContents($hostid)
  {
    $hostid = (int) $hostid;

    $query = "SELECT license  FROM Hosts "
        . " WHERE id = $hostid";

    $content = @sqlString($query, 'license');

    // not doing error()  here since it writes it to the file
    if($content == null || $content == ''){
      echo("Unable to retrieve license file.");
      return;
    }
    
    echo tag('result', tag('contents', $content, true));
  }
}
?>
